import numpy as np
import sys
import os
import h5py
import re
import xarray as xr #cite as: Hoyer, S. & Hamman, J., (2017). xarray: N-D labeled Arrays and Datasets in Python. Journal of Open Research Software. 5(1), p.10. DOI: http://doi.org/10.5334/jors.148
import gc
import glob
import copy
#from sklearn.neighbors import BallTree
import matplotlib.pyplot as plt
import netCDF4 as nv
from scipy.interpolate import interp1d,interp2d,griddata
from scipy.interpolate import RectBivariateSpline as rbs
from scipy.interpolate import RegularGridInterpolator as rgi
from scipy.optimize import brentq, curve_fit, minimize_scalar, minimize, Bounds
from scipy.stats import binned_statistic_2d, binned_statistic_dd, binned_statistic
import pickle
import cartopy.crs as ccrs
from matplotlib import rcParams

rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = False
rcParams['text.usetex'] = False
rcParams['font.sans-serif'] = ['cmr10', 'Times-Roman']
rcParams['font.weight'] = 'normal'
rcParams['font.size'] = 20
rcParams['xtick.labelsize'] = 20
rcParams['ytick.labelsize'] = 20
rcParams['axes.labelsize'] = 22
rcParams['axes.linewidth'] = 1
rcParams['axes.unicode_minus'] = False
rcParams['xtick.minor.width'] = 1
rcParams['ytick.minor.width'] = 1
rcParams['xtick.major.width'] = 1
rcParams['ytick.major.width'] = 1
rcParams['xtick.minor.size'] = 4
rcParams['ytick.minor.size'] = 4
rcParams['xtick.major.size'] = 5
rcParams['ytick.major.size'] = 5
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'
rcParams['mathtext.fontset'] = 'cm'


import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.colors as colors
from matplotlib import gridspec
from matplotlib import ticker
import matplotlib as mpl
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import NullFormatter
from collections import OrderedDict, Counter
from matplotlib.lines import Line2D
from matplotlib.patches import Patch, Rectangle, Ellipse
import matplotlib.patheffects as pe



def open_pickled_data(basepath='',basenames=['eta','meter'],zrange=1000):
  ATCE_q={}
  RTCE_q={}
  ATCE_t={}
  RTCE_t={}
  RTCE_p={}
  ATCE_p={}
  RTCE_e={}
  ATCE_e={}
  Npart={}
  zpart={}
  data={}
  datafile = open(basepath+basenames[0]+'data%i_600.pickle'%zrange,'rb')
  #datafile = open(basepath+basenames+'data%i_600.pickle'%zrange,'rb')
  data = pickle.load(datafile)
  datafile.close()
  ATCE_q[0]=data['ATCE_q']*1000.
  RTCE_q[0]=data['RTCE_q']*1000.
  ATCE_t[0]=data['ATCE_t']
  RTCE_t[0]=data['RTCE_t']
  RTCE_p[0]=data['RTCE_p']
  ATCE_p[0]=data['ATCE_p']
  RTCE_e[0]=data['RTCE_e']
  ATCE_e[0]=data['ATCE_e']
  Npart[0]=data['Npart']
  #zpart[0]=data['zpart']

  datafile = open(basepath+basenames[1]+'data%i_600.pickle'%zrange,'rb')
  data = pickle.load(datafile)
  datafile.close()
  ATCE_q[1]=data['ATCE_q']*1000.
  RTCE_q[1]=data['RTCE_q']*1000.
  ATCE_t[1]=data['ATCE_t']
  RTCE_t[1]=data['RTCE_t']
  RTCE_p[1]=data['RTCE_p']
  ATCE_p[1]=data['ATCE_p']
  RTCE_e[1]=data['RTCE_e']
  ATCE_e[1]=data['ATCE_e']
  Npart[1]=data['Npart']
  # zpart[1]=data['zpart']

  return ATCE_q,RTCE_q,ATCE_t,RTCE_t,ATCE_p,RTCE_p,ATCE_e,RTCE_e,Npart#,zpart

def plot_Figure1(
  length=288,
  basepath='',
  basenames=['output_eta_','output_meter_'],
  outputpath='./',
  basename='ATCE1',
  interptopo=[True,True],
  ls=['-','--','-.',':','-','--'],
  alpha=[1,1],
  labels=['$\\eta$','$z$'],
  one_release=False,
  zrange=[5000,10000],
  zrlab=['5-10 km','10-20 km','20-30 km','30-40 km','40-50 km']):

  s4 = ['jan','apr','jul','oct']

  kleur2=['indigo']*2#[plt.cm.tab20b(17/20.),plt.cm.tab20b(17/20.)]
  kleur=['slategray']*2#[plt.cm.tab20c(17/20.),plt.cm.tab20c(17/20.)]

  kleuren=['indigo','slategray']
  cmap=plt.cm.viridis
  t = np.linspace(0,48,length)
  fig,ax3=plt.subplots(1,3,figsize=(20,6))

  ax = {}
  ax_f = {}
  for j in range(3):
    inner = gridspec.GridSpecFromSubplotSpec(90, 1, subplot_spec=ax3[j], wspace=0, hspace=0)
    ax[j] = plt.Subplot(fig, inner[0:])
    # ax_f[i,j] = plt.Subplot(fig, inner[60:])
    ax3[j].axis('off')
    fig.add_subplot(ax[j])
    # fig.add_subplot(ax_f[i, j])

    for l in [9,21,33,45]:
      ax[j].axvspan(l,l+1,color='lavender',alpha=0.5,lw=0.5,zorder=0)
      # ax_f[i,j].axvline(l,color='k',ls='--',alpha=0.5,lw=0.5)

  mm=0
  for zr in zrange:
    ATCE_q={}
    RTCE_q={}
    ATCE_t={}
    RTCE_t={}
    ATCE_p={}
    RTCE_p={}
    ATCE_e={}
    RTCE_e={}
    for ii in range(len(s4)):
      basename_temp=['','']
      for ll in range(len(basenames)):
        basename_temp[ll] = basenames[ll]+s4[ii]
      ATCE_q[ii],RTCE_q[ii],ATCE_t[ii],RTCE_t[ii],ATCE_p[ii],RTCE_p[ii],ATCE_e[ii],RTCE_e[ii],Npart=open_pickled_data(basepath=basepath,
        basenames=basename_temp,zrange=zr)

      if ii==0:
        for j in range(2):
          ATCE_q[len(s4)+j]=ATCE_q[ii][j]
          RTCE_q[len(s4)+j]=RTCE_q[ii][j]
          ATCE_t[len(s4)+j]=ATCE_t[ii][j]
          RTCE_t[len(s4)+j]=RTCE_t[ii][j]
          ATCE_p[len(s4)+j]=ATCE_p[ii][j]
          RTCE_p[len(s4)+j]=RTCE_p[ii][j]
          ATCE_e[len(s4)+j]=ATCE_e[ii][j]
          RTCE_e[len(s4)+j]=RTCE_e[ii][j]
      else:
        for j in range(2):
          ATCE_q[len(s4)+j]=ATCE_q[len(s4)+j]+ATCE_q[ii][j]
          RTCE_q[len(s4)+j]=RTCE_q[len(s4)+j]+RTCE_q[ii][j]
          ATCE_t[len(s4)+j]=ATCE_t[len(s4)+j]+ATCE_t[ii][j]
          RTCE_t[len(s4)+j]=RTCE_t[len(s4)+j]+RTCE_t[ii][j]
          ATCE_p[len(s4)+j]=ATCE_p[len(s4)+j]+ATCE_p[ii][j]
          RTCE_p[len(s4)+j]=RTCE_p[len(s4)+j]+RTCE_p[ii][j]
          ATCE_e[len(s4)+j]=ATCE_e[len(s4)+j]+ATCE_e[ii][j]
          RTCE_e[len(s4)+j]=RTCE_e[len(s4)+j]+RTCE_e[ii][j]

    for j in range(2):
      ATCE_q[len(s4)+j]=ATCE_q[len(s4)+j]/len(s4)
      RTCE_q[len(s4)+j]=RTCE_q[len(s4)+j]/len(s4)
      ATCE_t[len(s4)+j]=ATCE_t[len(s4)+j]/len(s4)
      RTCE_t[len(s4)+j]=RTCE_t[len(s4)+j]/len(s4)
      ATCE_p[len(s4)+j]=ATCE_p[len(s4)+j]/len(s4)
      RTCE_p[len(s4)+j]=RTCE_p[len(s4)+j]/len(s4)
      ATCE_e[len(s4)+j]=ATCE_e[len(s4)+j]/len(s4)
      RTCE_e[len(s4)+j]=RTCE_e[len(s4)+j]/len(s4)

    lw=2.5

    for j in range(2):
      ax[0].plot(t,ATCE_p[len(s4)+j][:],color=kleuren[mm],ls=ls[j],lw=lw,alpha=alpha[0])
      ax[1].plot(t,ATCE_t[len(s4)+j][:],color=kleuren[mm],ls=ls[j],lw=lw,alpha=alpha[0])
      if (j==0):
        ax[2].plot(t,ATCE_q[len(s4)+j][:],color=kleuren[mm],ls=ls[j],lw=lw,alpha=alpha[0],label=zrlab[mm])
      else:
        ax[2].plot(t,ATCE_q[len(s4)+j][:],color=kleuren[mm],ls=ls[j],lw=lw,alpha=alpha[0])

    mlab=['Jan','Apr','Jul','Oct']
    iii=0
    for ii in range(len(s4)):
      for j in range(2):
        ax[0].plot(t,ATCE_p[ii][j][:],color=kleuren[mm],ls=ls[j],lw=1,alpha=0.3)
        ax[1].plot(t,ATCE_t[ii][j][:],color=kleuren[mm],ls=ls[j],lw=1,alpha=0.3)#,label=mlab[iii])
        ax[2].plot(t,ATCE_q[ii][j][:],color=kleuren[mm],ls=ls[j],lw=1,alpha=0.3)
      iii+=1     
    print(Npart)
    mm=mm+1

    tags=np.array(['(a)', '(b)', '(c)'])
    for i in range(3):
      ax[i].text(0.99,.99,tags[i],horizontalalignment='right',verticalalignment='top',transform=ax[i].transAxes,color='k',
        fontsize=20,bbox=dict(facecolor='white',edgecolor='none',alpha=0.7,boxstyle='round',pad=0.01))

    a=np.array([ 0,  1,  2,  3,  4,  5,  6,  7, 8, 10, 11, 12, 13, 14, 15, 16, 17, 18,
         19,20, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 34, 35, 36, 37,
         38, 39, 40, 41, 42, 43,44, 46]).astype(int)

    if zr==5000:
      cc=kleur2
      pos2=0.98
      pos1=0.11
      rot0=8.3
      rot1=20
      rot2=3
      lab1='%i-%i km'%(5,10)
    else:
      cc=kleur
      pos2=0.89
      pos1=0.03
      rot0=10
      rot1=26.4
      rot2=3.5
      lab1='%i-%i km'%(10,20)

    ax[0].set_ylabel('ATCE(PV) [pvu]')

    diff1 = (ATCE_p[len(s4)+1][6*48-1]-ATCE_p[len(s4)][6*48-1])/ATCE_p[len(s4)+1][6*48-1]
    assmm = 0
    assme = 0
    for l in range(48):
      if l not in a: 
        assmm = assmm + (ATCE_p[len(s4)+1][l*6]-ATCE_p[len(s4)+1][l*6-6])
        assme = assme + (ATCE_p[len(s4)][l*6]-ATCE_p[len(s4)][l*6-6])
    diff2 = (ATCE_p[len(s4)+1][6*48-1]-assme-ATCE_p[len(s4)][6*48-1]+assmm)/(ATCE_p[len(s4)+1][6*48-1]-assmm)

    ax[0].text(.5, .9, 'Av. improv.: %2.1f%% (%2.1f%%)' %(diff2*100,diff1*100), fontsize=20, 
      transform=ax[0].transAxes, color=cc[0],
          position=(0.02, pos2), fontweight='extra bold', ha='left', va='top')
    print('ATCE(PV) eta-meter/meter',zr, (ATCE_p[len(s4)+1][6*48-1]-ATCE_p[len(s4)][6*48-1])/ATCE_p[len(s4)+1][6*48-1]*100,assmm,assme,ATCE_p[len(s4)+1][6*48-1])

    ax[1].set_ylabel('ATCE$(\\theta)$ [K]')

    diff1 = (ATCE_t[len(s4)+1][6*48-1]-ATCE_t[len(s4)][6*48-1])/ATCE_t[len(s4)+1][6*48-1]
    assmm = 0
    assme = 0
    for l in range(48):
      if l not in a: 
        assmm = assmm + (ATCE_t[len(s4)+1][l*6]-ATCE_t[len(s4)+1][l*6-6])
        assme = assme + (ATCE_t[len(s4)][l*6]-ATCE_t[len(s4)][l*6-6])
    diff2 = (ATCE_t[len(s4)+1][6*48-1]-assme-ATCE_t[len(s4)][6*48-1]+assmm)/(ATCE_t[len(s4)+1][6*48-1]-assmm)

    ax[1].text(.5, .9, 'Av. improv.: %2.1f%% (%2.1f%%)' %(diff2*100,diff1*100), fontsize=20, 
      transform=ax[1].transAxes, color=cc[0],
          position=(0.02, pos2), fontweight='extra bold', ha='left', va='top')
    print('ATCE(PV) eta-meter/meter', (ATCE_t[len(s4)+1][6*48-1]-ATCE_t[len(s4)][6*48-1])/ATCE_t[len(s4)+1][6*48-1]*100)

    ax[2].set_ylabel('ATCE(q) [g kg$^{-1}$]')#' [$10^{-%i}$]' %pp)

    diff1 = (ATCE_q[len(s4)+1][6*48-1]-ATCE_q[len(s4)][6*48-1])/ATCE_q[len(s4)+1][6*48-1]
    assmm = 0
    assme = 0
    for l in range(48):
      if l not in a: 
        assmm = assmm + (ATCE_q[len(s4)+1][l*6]-ATCE_q[len(s4)+1][l*6-6])
        assme = assme + (ATCE_q[len(s4)][l*6]-ATCE_q[len(s4)][l*6-6])
    diff2 = (ATCE_q[len(s4)+1][6*48-1]-assme-ATCE_q[len(s4)][6*48-1]+assmm)/(ATCE_q[len(s4)+1][6*48-1]-assmm)

    ax[2].text(.5, .9, 'Av. improv.: %2.1f%% (%2.1f%%)' %(diff2*100,diff1*100), fontsize=20, 
      transform=ax[2].transAxes, color=cc[0],
          position=(0.02, pos2), fontweight='extra bold', ha='left', va='top')
    print('ATCE(PV) eta-meter/meter', (ATCE_q[len(s4)+1][6*48-1]-ATCE_q[len(s4)][6*48-1])/ATCE_q[len(s4)+1][6*48-1]*100)

  for i in range(3):
    ax[i].set_xlabel('Travel time [hours]')

  ax[0].set_ylabel('ATCE(PV) [pvu]')

  ax[1].set_ylabel('ATCE$(\\theta)$ [K]')

  for j in range(3):
    if not (j==2):
      ax[j].set_ylim(bottom=0)
    else:
      ax[j].set_yscale('log')
      ax[j].set_ylim(top=1.1)
      ax[j].set_ylim(bottom=1e-5)
    ax[j].set_xlim(left=0,right=48)

  line1 = Line2D([0],[0],lw=lw,ls=ls[0],color='dimgrey',label=labels[0])
  line2 = Line2D([0],[0],lw=lw,ls=ls[1],color='dimgrey',label=labels[1])
  for j in range(3):
    ax[j].legend(handles=[line1,line2],handlelength=2,frameon=False,loc='lower right',fontsize=22)

  ax[1].legend(frameon=False,loc='upper left')
  ax[2].legend(frameon=False,loc='lower right')
  fig.subplots_adjust(top=0.96, bottom=0.13, left=0.06, right=0.99, wspace=0.2, hspace=0.1)

  plt.savefig(outputpath+basename+'_q.png')
  plt.savefig(outputpath+basename+'_q.pdf')
  plt.close()
