import numpy as np
import sys
import os
import h5py
import re
import eccodes
import cfgrib
import xarray as xr #cite as: Hoyer, S. & Hamman, J., (2017). xarray: N-D labeled Arrays and Datasets in Python. Journal of Open Research Software. 5(1), p.10. DOI: http://doi.org/10.5334/jors.148
import gc
import glob
import copy
#from sklearn.neighbors import BallTree
import matplotlib.pyplot as plt
import netCDF4 as nv
from scipy.interpolate import interp1d,interp2d,griddata
from scipy.interpolate import RectBivariateSpline as rbs
from scipy.interpolate import RegularGridInterpolator as rgi
from scipy.optimize import brentq, curve_fit, minimize_scalar, minimize, Bounds
from scipy.stats import binned_statistic_2d, binned_statistic_dd, binned_statistic
from sklearn.neighbors import BallTree
import pandas as pa
import multiprocessing as mp
from mpl_toolkits.axes_grid1 import make_axes_locatable

import cartopy.crs as ccrs
from matplotlib import rcParams

rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = False
rcParams['text.usetex'] = False
rcParams['font.sans-serif'] = ['cmr10', 'Times-Roman']
rcParams['font.weight'] = 'normal'
rcParams['font.size'] = 20
rcParams['xtick.labelsize'] = 20
rcParams['ytick.labelsize'] = 20
rcParams['axes.labelsize'] = 22
rcParams['axes.linewidth'] = 1
rcParams['axes.unicode_minus'] = False
rcParams['xtick.minor.width'] = 1
rcParams['ytick.minor.width'] = 1
rcParams['xtick.major.width'] = 1
rcParams['ytick.major.width'] = 1
rcParams['xtick.minor.size'] = 4
rcParams['ytick.minor.size'] = 4
rcParams['xtick.major.size'] = 5
rcParams['ytick.major.size'] = 5
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'
rcParams['mathtext.fontset'] = 'cm'


import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.colors as colors
from matplotlib import gridspec
from matplotlib import ticker
import matplotlib as mpl
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import NullFormatter
from collections import OrderedDict, Counter
from matplotlib.lines import Line2D
from matplotlib.patches import Patch, Rectangle, Ellipse
import matplotlib.patheffects as pe
import pickle 


def plot_density_pickle(z=[0,1e5], fname='density_profile'):

  both=True
  #grid from sea level
  grid_xlon = np.arange(0,360,0.5)
  grid_ylat = np.arange(-90.,90.5,0.5)
  grid_z = np.concatenate([np.array([0.,25.,50.,75.]),np.logspace(2,np.log10(z[1]),100)])

  #zpos = np.logspace(np.log10(grid_z[0]+(grid_z[1]-grid_z[0])/2.),np.log10(grid_z[-2]+(grid_z[-1]-grid_z[-2])/2.),99)
  zpos=grid_z[:-1]+(grid_z[1:]-grid_z[:-1])/2.

  poles_pickle=open('/home/lucie/conservation/data/poles.pickle', "rb")
  midlat_pickle=open('/home/lucie/conservation/data/midlat.pickle', "rb")
  tropics_pickle=open('/home/lucie/conservation/data/topics.pickle', "rb")

  poles = pickle.load(poles_pickle)
  midlat = pickle.load(midlat_pickle)
  tropics = pickle.load(tropics_pickle)
  poles_pickle.close()
  midlat_pickle.close()
  tropics_pickle.close()

  polestr=poles['polestr']
  polespbl=poles['polespbl']
  poleseta=poles['poleseta']
  ec_poleseta=poles['ec_poleseta']
  polesmeter=poles['polesmeter']
  ec_polesmeter=poles['ec_polesmeter']
  polestopo=poles['polestopo']

  midlattr=midlat['midlattr']
  midlatpbl=midlat['midlatpbl']
  midlateta=midlat['midlateta']
  ec_midlateta=midlat['ec_midlateta']
  midlatmeter=midlat['midlatmeter']
  ec_midlatmeter=midlat['ec_midlatmeter']
  midlattopo=midlat['midlattopo']

  troptr=tropics['troptr']
  troppbl=tropics['troppbl']
  tropeta=tropics['tropeta']
  ec_tropeta=tropics['ec_tropeta']
  tropmeter=tropics['tropmeter']
  ec_tropmeter=tropics['ec_tropmeter']
  troptopo=tropics['troptopo']

  dpoleseta=np.abs(poleseta-ec_poleseta)
  dpolesmeter=np.abs(polesmeter-ec_polesmeter)
  dmidlateta=np.abs(midlateta-ec_midlateta)
  dmidlatmeter=np.abs(midlatmeter-ec_midlatmeter)
  dtropeta=np.abs(tropeta-ec_tropeta)
  dtropmeter=np.abs(tropmeter-ec_tropmeter)

  fig,ax=plt.subplots(1,figsize=(16,9),sharex=True,sharey=True)
  ax.axis('off')

  if (both==True):
    istep_p1=8
    istep_p2=6
  else:
    istep_p1=9
    istep_p2=5
  inner = gridspec.GridSpecFromSubplotSpec(20, 45, subplot_spec=ax, wspace=0, hspace=0)
  ax1a = fig.add_subplot(inner[0:18, 0:istep_p1])
  ax1b = fig.add_subplot(inner[0:18, istep_p1:istep_p1+istep_p2])
  ax2a = fig.add_subplot(inner[0:18, istep_p1+istep_p2+1:istep_p1*2+istep_p2+1])
  ax2b = fig.add_subplot(inner[0:18, istep_p1*2+istep_p2+1:istep_p1*2+istep_p2*2+1])
  ax3a = fig.add_subplot(inner[0:18, istep_p1*2+istep_p2*2+2:istep_p1*3+istep_p2*2+2])
  ax3b = fig.add_subplot(inner[0:18, istep_p1*3+istep_p2*2+2:istep_p1*3+istep_p2*3+2])

  ax1ac = fig.add_subplot(inner[18:, 0:istep_p1])
  ax1bc = fig.add_subplot(inner[18:, istep_p1:istep_p1+istep_p2])
  ax2ac = fig.add_subplot(inner[18:, istep_p1+istep_p2+1:istep_p1*2+istep_p2+1])
  ax2bc = fig.add_subplot(inner[18:, istep_p1*2+istep_p2+1:istep_p1*2+istep_p2*2+1])
  ax3ac = fig.add_subplot(inner[18:, istep_p1*2+istep_p2*2+2:istep_p1*3+istep_p2*2+2])
  ax3bc = fig.add_subplot(inner[18:, istep_p1*3+istep_p2*2+2:istep_p1*3+istep_p2*3+2])

  kleur3=plt.cm.tab20b(3/20.)
  kleur3a=plt.cm.tab20b(14/20.)
  kleur2=plt.cm.tab20b(3/20.)
  kleur2a=plt.cm.tab20b(14/20.)
  kleur1=plt.cm.tab20b(3/20.)
  kleur1a=plt.cm.tab20b(14/20.)

  ax1a.plot(ec_poleseta,zpos,ls='-',lw=1,color='k',zorder=3)
  ax1a.plot(poleseta,zpos,ls='-',lw=3,color=kleur1,zorder=1)
  ax1ac.plot(ec_poleseta,zpos,ls='-',lw=1,color='k',zorder=3)
  ax1ac.plot(poleseta,zpos,ls='-',lw=3,color=kleur1,zorder=1)
  ax1a.axhline(polespbl,ls='-',lw=5,color='lavender',zorder=0)
  ax1b.axhline(polespbl,ls='-',lw=5,color='lavender',zorder=0)
  ax1a.axhline(polestr,ls='--',lw=5,color='lavender',zorder=0)
  ax1b.axhline(polestr,ls='--',lw=5,color='lavender',zorder=0)
  ax1a.text(0,polespbl*0.9, 'ABL',
      fontsize=23,
      color='lavender',
      rotation=0,
      fontweight='extra bold', ha='left', va='top')
  ax1a.text(0,polestr*0.9, 'Tropopause',
      fontsize=23,
      color='lavender',
      rotation=0,
      fontweight='extra bold', ha='left', va='top',zorder=0)
  #ax1a.fill_between([0,np.max(poleseta)],polestopo,color='lavender',zorder=0,interpolate=True)
  et=(dpoleseta)
  me=(dpolesmeter)
  #ax1a.plot(ec_polesmeter,zpos,ls='--',lw=1,color='k',zorder=2)
  ax1a.plot(polesmeter,zpos,ls='--',lw=3,color=kleur1a,zorder=2)
  ax1ac.plot(polesmeter,zpos,ls='--',lw=3,color=kleur1a,zorder=2)
  if (both==True):
    ax1b.fill_betweenx(zpos,np.abs(et),0,color=kleur1,zorder=1)
    ax1b.fill_betweenx(zpos,np.abs(me),0,color=kleur1a,zorder=2)
    ax1b.plot(np.abs(et),zpos,color=kleur1,zorder=3)
    ax1b.plot(np.abs(me),zpos,color=kleur1a,zorder=3)
    ax1bc.fill_betweenx(zpos,np.abs(et),0,color=kleur1,zorder=1)
    ax1bc.fill_betweenx(zpos,np.abs(me),0,color=kleur1a,zorder=2)
    ax1bc.plot(np.abs(et),zpos,color=kleur1,zorder=3)
    ax1bc.plot(np.abs(me),zpos,color=kleur1a,zorder=3)
    ax1b.set_xlim([0,0.07])
  else:
    ax1b.fill_betweenx(zpos,np.abs(et-me),0,where=et-me>0,color=kleur1)#'lightcoral')
    ax1b.fill_betweenx(zpos,np.abs(et-me),0,where=et-me<0,color=kleur1a)#'yellowgreen')
    ax1bc.fill_betweenx(zpos,np.abs(et-me),0,where=et-me>0,color=kleur1)#'lightcoral')
    ax1bc.fill_betweenx(zpos,np.abs(et-me),0,where=et-me<0,color=kleur1a)#'yellowgreen')
    ax1b.set_xlim([0,0.06])
  ax1bc.set_xlim(ax1b.get_xlim())
  ax1ac.set_xlim(ax1a.get_xlim())

  ax1a.set_yscale('log')
  ax1b.set_yscale('log')

  ax2a.plot(ec_midlateta,zpos,ls='-',lw=1,color='k',zorder=3)
  ax2a.plot(midlateta,zpos,ls='-',lw=3,color=kleur2,zorder=1)
  ax2ac.plot(ec_midlateta,zpos,ls='-',lw=1,color='k',zorder=3)
  ax2ac.plot(midlateta,zpos,ls='-',lw=3,color=kleur2,zorder=1)
  ax2a.axhline(midlatpbl,ls='-',lw=5,color='lavender',zorder=0)
  ax2b.axhline(midlatpbl,ls='-',lw=5,color='lavender',zorder=0)
  ax2a.axhline(midlattr,ls='--',lw=5,color='lavender',zorder=0)
  ax2b.axhline(midlattr,ls='--',lw=5,color='lavender',zorder=0)
  ax2a.text(0,midlatpbl*0.9, 'ABL',
      fontsize=23,
      color='lavender',
      rotation=0,
      fontweight='extra bold', ha='left', va='top')
  ax2a.text(0,midlattr*0.9, 'Tropopause',
      fontsize=23,
      color='lavender',
      rotation=0,
      fontweight='extra bold', ha='left', va='top',zorder=0)

  et=(dmidlateta)
  me=(dmidlatmeter)
  #ax2a.plot(ec_midlatmeter,zpos,ls='--',lw=1,color='k',zorder=2)
  ax2a.plot(midlatmeter,zpos,ls='--',lw=3,color=kleur2a,zorder=2)
  ax2ac.plot(midlatmeter,zpos,ls='--',lw=3,color=kleur2a,zorder=2)
  if (both==True):
    ax2b.fill_betweenx(zpos,np.abs(et),0,color=kleur1,zorder=1)
    ax2b.fill_betweenx(zpos,np.abs(me),0,color=kleur1a,zorder=2)
    ax2b.plot(np.abs(et),zpos,color=kleur1,zorder=3)
    ax2b.plot(np.abs(me),zpos,color=kleur1a,zorder=3)
    ax2bc.fill_betweenx(zpos,np.abs(et),0,color=kleur1,zorder=1)
    ax2bc.fill_betweenx(zpos,np.abs(me),0,color=kleur1a,zorder=2)
    ax2bc.plot(np.abs(et),zpos,color=kleur1,zorder=3)
    ax2bc.plot(np.abs(me),zpos,color=kleur1a,zorder=3)
    ax2b.set_xlim([0,0.07])
  else:
    ax2b.fill_betweenx(zpos,np.abs(et-me),0,where=et-me>0,color=kleur2)#'lightcoral')
    ax2b.fill_betweenx(zpos,np.abs(et-me),0,where=et-me<0,color=kleur2a)#'yellowgreen')
    ax2bc.fill_betweenx(zpos,np.abs(et-me),0,where=et-me>0,color=kleur2)#'lightcoral')
    ax2bc.fill_betweenx(zpos,np.abs(et-me),0,where=et-me<0,color=kleur2a)#'yellowgreen')
    ax2b.set_xlim([0,0.06])
  ax2bc.set_xlim(ax2b.get_xlim())
  ax2ac.set_xlim(ax2a.get_xlim())

  ax2a.set_yscale('log')
  ax2b.set_yscale('log')

  ax3a.plot(ec_tropeta,zpos,ls='-',lw=1,color='k',zorder=3)
  ax3a.plot(tropeta,zpos,ls='-',lw=3,color=kleur3,zorder=1)
  ax3ac.plot(ec_tropeta,zpos,ls='-',lw=1,color='k',zorder=3)
  ax3ac.plot(tropeta,zpos,ls='-',lw=3,color=kleur3,zorder=1)
  ax3a.axhline(troppbl,ls='-',lw=5,color='lavender',zorder=0)

  ax3a.text(0,troppbl*0.9, 'ABL',
      fontsize=23,
      color='lavender',
      rotation=0,
      fontweight='extra bold', ha='left', va='top')
  ax3b.axhline(troppbl,ls='-',lw=5,color='lavender',zorder=0)
  ax3a.axhline(troptr,ls='--',lw=5,color='lavender',zorder=0)
  ax3a.text(0,troptr*0.9, 'Tropopause',
      fontsize=23,
      color='lavender',
      rotation=0,
      fontweight='extra bold', ha='left', va='top',zorder=0)
  ax3b.axhline(troptr,ls='--',lw=5,color='lavender',zorder=0)
  et=(dtropeta)
  me=(dtropmeter)
  #ax3a.plot(ec_tropmeter,zpos,ls='--',lw=4,color='k',zorder=3)
  ax3a.plot(tropmeter,zpos,ls='--',lw=3,color=kleur3a,zorder=2)
  ax3ac.plot(tropmeter,zpos,ls='--',lw=3,color=kleur3a,zorder=2)
  if (both==True):
    ax3b.fill_betweenx(zpos,np.abs(et),0,color=kleur1,zorder=1)
    ax3b.fill_betweenx(zpos,np.abs(me),0,color=kleur1a,zorder=2)
    ax3b.plot(np.abs(et),zpos,color=kleur1,zorder=3)
    ax3b.plot(np.abs(me),zpos,color=kleur1a,zorder=3)
    ax3bc.fill_betweenx(zpos,np.abs(et),0,color=kleur1,zorder=1)
    ax3bc.fill_betweenx(zpos,np.abs(me),0,color=kleur1a,zorder=2)
    ax3bc.plot(np.abs(et),zpos,color=kleur1,zorder=3)
    ax3bc.plot(np.abs(me),zpos,color=kleur1a,zorder=3)
    ax3b.set_xlim([0,0.07])
  else:
    ax3b.fill_betweenx(zpos,np.abs(et-me),0,where=et-me>0,color=kleur3,label='$\\Delta_{\\eta}>\\Delta_{z}$')#'lightcoral')
    ax3b.fill_betweenx(zpos,np.abs(et-me),0,where=et-me<0,color=kleur3a,label='$\\Delta_{\\eta}<\\Delta_{z}$')#'yellowgreen')
    ax3bc.fill_betweenx(zpos,np.abs(et-me),0,where=et-me>0,color=kleur3,label='$\\Delta_{\\eta}>\\Delta_{z}$')#'lightcoral')
    ax3bc.fill_betweenx(zpos,np.abs(et-me),0,where=et-me<0,color=kleur3a,label='$\\Delta_{\\eta}<\\Delta_{z}$')#'yellowgreen')
    ax3b.set_xlim([0,0.06])
  ax3bc.set_xlim(ax3b.get_xlim())
  ax3ac.set_xlim(ax3a.get_xlim())

  tags=np.array(['(a)', '(b)', '(c)'])
  ax1b.text(0.99,.99,tags[0],horizontalalignment='right',verticalalignment='top',transform=ax1b.transAxes,color='k',fontsize=22,
    bbox=dict(facecolor='white',edgecolor='none',alpha=0.7,boxstyle='round',pad=0.01))
  ax2b.text(0.99,.99,tags[1],horizontalalignment='right',verticalalignment='top',transform=ax2b.transAxes,color='k',fontsize=22,
    bbox=dict(facecolor='white',edgecolor='none',alpha=0.7,boxstyle='round',pad=0.01))
  ax3b.text(0.99,.99,tags[2],horizontalalignment='right',verticalalignment='top',transform=ax3b.transAxes,color='k',fontsize=22,
    bbox=dict(facecolor='white',edgecolor='none',alpha=0.7,boxstyle='round',pad=0.01))

  miny=100.
  ax3a.set_ylim([miny,5e4])
  ax3b.set_ylim([miny,5e4])
  ax1a.set_ylim([miny,5e4])
  ax1b.set_ylim([miny,5e4])
  ax2a.set_ylim([miny,5e4])
  ax2b.set_ylim([miny,5e4])

  ax3ac.set_ylim([0,miny])
  ax3bc.set_ylim([0,miny])
  ax1ac.set_ylim([0,miny])
  ax1bc.set_ylim([0,miny])
  ax2ac.set_ylim([0,miny])
  ax2bc.set_ylim([0,miny])


  ax3a.set_yscale('log')
  ax3b.set_yscale('log')
  #ax3b.set_xscale('log')

  ax1a.text(0.9,.98,'$|\\rm{lat}|>66^{\\circ}$',horizontalalignment='right',verticalalignment='top',
    transform=ax1a.transAxes,color='k')#kleur1)
  ax2a.text(0.9,.98,'$66^{\\circ}>|\\rm{lat}|>23^{\\circ}$',horizontalalignment='right',verticalalignment='top',
    transform=ax2a.transAxes,color='k')#kleur2)
  ax3a.text(0.9,.98,'$|\\rm{lat}|<23^{\\circ}$',horizontalalignment='right',verticalalignment='top',
    transform=ax3a.transAxes,color='k')#kleur3)

  line1 = Line2D([0],[0],ls='-',lw=3,color=kleur3,label='$\\rho_{\\eta }$')
  line2 = Line2D([0],[0],ls='--',lw=3,color=kleur3a,label='$\\rho_{z }$')
  line3 = Line2D([0],[0],ls='-',lw=1,color='k',label='$\\rho_{\\rm ERA5}$')

  if both==True:
    ax3a.legend(handles=[line1,line2,line3],handlelength=2,frameon=False,loc='lower left',fontsize=20)
  else:
    ax2a.legend(handles=[line1,line2,line3],handlelength=2,frameon=False,loc='lower left',fontsize=18)
    patch1 = Patch(facecolor=kleur3,edgecolor=kleur3,label='$\\Delta_{\\eta}>\\Delta_{z}$')
    patch2 = Patch(facecolor=kleur3a,edgecolor=kleur3a,label='$\\Delta_{\\eta}<\\Delta_{z}$')
    ax3a.legend(handles=[patch1,patch2],frameon=False,loc='lower left',fontsize=18)
  
  ax1b.set_yticklabels([])
  ax2b.set_yticklabels([])
  ax3b.set_yticklabels([])
  ax2bc.set_yticks([0,50])
  ax3bc.set_yticks([0,50])
  ax1bc.set_yticks([0,50])
  ax1bc.set_yticklabels([])
  ax2bc.set_yticklabels([])
  ax3bc.set_yticklabels([])
  ax2ac.set_yticks([0,50])
  ax2ac.set_yticklabels([])
  ax3ac.set_yticks([0,50])
  ax3ac.set_yticklabels([])
  ax1ac.set_yticklabels([0,50])
  ax1ac.set_yticks([0,50])
  ax2a.set_yticklabels([])
  ax3a.set_yticklabels([])
  ax1a.set_ylabel('Height (m)')
  ax1a.set_xticklabels([])
  ax2a.set_xticklabels([])
  ax3a.set_xticklabels([])
  ax1b.set_xticklabels([])
  ax2b.set_xticklabels([])
  ax3b.set_xticklabels([])
  ax1ac.set_xlabel('Density (kg m$^{-3}$)')
  ax2ac.set_xlabel('Density (kg m$^{-3}$)')
  ax3ac.set_xlabel('Density (kg m$^{-3}$)')
  if both==True:
    ax1bc.set_xlabel('$|\\rho_{x}-\\rho_{\\rm ERA5}|$ \n(kg m$^{-3}$)',fontsize=18)
    ax2bc.set_xlabel('$|\\rho_{x}-\\rho_{\\rm ERA5}|$ \n(kg m$^{-3}$)',fontsize=18)
    ax3bc.set_xlabel('$|\\rho_{x}-\\rho_{\\rm ERA5}|$ \n(kg m$^{-3}$)',fontsize=18)
  else:
    ax1bc.set_xlabel('$|\\Delta_{\\eta}-\\Delta_{z}|$',fontsize=18)
    ax2bc.set_xlabel('$|\\Delta_{\\eta}-\\Delta_{z}|$',fontsize=18)
    ax3bc.set_xlabel('$|\\Delta_{\\eta}-\\Delta_{z}|$',fontsize=18)
  fig.subplots_adjust(top=0.96, bottom=0.14, left=0.08, right=0.98, wspace=0.15, hspace=0.02)

  plt.savefig('/home/lucie/FP11/'+fname+'.png')
  plt.savefig('/home/lucie/FP11/'+fname+'.pdf')
  plt.close()

  ipbl=np.abs(zpos-midlatpbl).argmin()
  print('midlat PBL:', np.sum(dmidlateta[1:ipbl]/ec_midlatmeter[1:ipbl]*zpos[1:ipbl])/np.sum(zpos[1:ipbl])*100,
        np.sum(dmidlatmeter[1:ipbl]/ec_midlatmeter[1:ipbl]*zpos[1:ipbl])/np.sum(zpos[1:ipbl])*100)
  print('midlat PBL max:', np.max(dmidlateta[1:ipbl]/ec_midlatmeter[1:ipbl])*100,
        np.max(dmidlatmeter[1:ipbl]/ec_midlatmeter[1:ipbl])*100)

  itr=np.abs(zpos-midlattr).argmin()
  print('midlat tropo:', np.sum(dmidlateta[ipbl:itr]/ec_midlatmeter[ipbl:itr]*zpos[ipbl:itr])/np.sum(zpos[ipbl:itr])*100,
        np.sum(dmidlatmeter[ipbl:itr]/ec_midlatmeter[ipbl:itr]*zpos[ipbl:itr])/np.sum(zpos[ipbl:itr])*100)
  print('midlat tropo max:', np.max(dmidlateta[ipbl:itr]/ec_midlatmeter[ipbl:itr])*100,
        np.max(dmidlatmeter[ipbl:itr]/ec_midlatmeter[ipbl:itr])*100)

  istr=np.abs(zpos-5e4).argmin()
  print('midlat strat:', np.sum(dmidlateta[itr:istr]/ec_midlatmeter[itr:istr]*zpos[itr:istr])/np.sum(zpos[itr:istr])*100,
        np.sum(dmidlatmeter[itr:istr]/ec_midlatmeter[itr:istr]*zpos[itr:istr])/np.sum(zpos[itr:istr])*100)
  print('midlat strat max:', np.max(dmidlateta[itr:istr]/ec_midlatmeter[itr:istr])*100,
        np.max(dmidlatmeter[itr:istr]/ec_midlatmeter[itr:istr])*100)
  print('PBL','Tropo','Strato', zpos[ipbl],zpos[itr],zpos[istr])

  ipbl=np.abs(zpos-polespbl).argmin()
  print('poles PBL:',np.sum(dpoleseta[1:ipbl]/ec_polesmeter[1:ipbl]*zpos[1:ipbl])/np.sum(zpos[1:ipbl])*100,
        np.sum(dpolesmeter[1:ipbl]/ec_polesmeter[1:ipbl]*zpos[1:ipbl])/np.sum(zpos[1:ipbl])*100)
  print('poles PBL max:',np.max(dpoleseta[1:ipbl]/ec_polesmeter[1:ipbl])*100,
      np.max(dpolesmeter[1:ipbl]/ec_polesmeter[1:ipbl])*100)
  i1km=np.abs(zpos-1000).argmin()
  print('poles 1km:',np.sum(dpoleseta[1:i1km]/ec_polesmeter[1:i1km]*zpos[1:i1km])/np.sum(zpos[1:i1km])*100,
        np.sum(dpolesmeter[1:i1km]/ec_polesmeter[1:i1km]*zpos[1:i1km])/np.sum(zpos[1:i1km])*100)
  print('poles 1km max:',np.max(dpoleseta[1:i1km]/ec_polesmeter[1:i1km])*100,
      np.max(dpolesmeter[1:i1km]/ec_polesmeter[1:i1km])*100)

  itr=np.abs(zpos-polestr).argmin()
  print('poles tropo:', np.sum(dpoleseta[ipbl:itr]/ec_polesmeter[ipbl:itr]*zpos[ipbl:itr])/np.sum(zpos[ipbl:itr])*100,
        np.sum(dpolesmeter[ipbl:itr]/ec_polesmeter[ipbl:itr]*zpos[ipbl:itr])/np.sum(zpos[ipbl:itr])*100)
  print('poles tropo max:', np.max(dpoleseta[ipbl:itr]/ec_polesmeter[ipbl:itr])*100,
        np.max(dpolesmeter[ipbl:itr]/ec_polesmeter[ipbl:itr])*100)

  istr=np.abs(zpos-5e4).argmin()
  print('poles strat:', np.sum(dpoleseta[itr:istr]/ec_polesmeter[itr:istr]*zpos[itr:istr])/np.sum(zpos[itr:istr])*100,
        np.sum(dpolesmeter[itr:istr]/ec_polesmeter[itr:istr]*zpos[itr:istr])/np.sum(zpos[itr:istr])*100)
  print('poles strat max:', np.max(dpoleseta[itr:istr]/ec_polesmeter[itr:istr])*100,
        np.max(dpolesmeter[itr:istr]/ec_polesmeter[itr:istr])*100)
  ipeak = np.argmax(dpolesmeter[itr:istr])
  print('poles strat peak meter:', (dpolesmeter[itr:istr]/ec_polesmeter[itr:istr])[ipeak]*100)
  print('PBL','Tropo','Strato', zpos[ipbl],zpos[itr],zpos[istr])

  ipbl=np.abs(zpos-troppbl).argmin()
  print('tropics PBL:',np.sum(dtropeta[1:ipbl]/ec_tropmeter[1:ipbl]*zpos[1:ipbl])/np.sum(zpos[1:ipbl])*100,
        np.sum(dtropmeter[1:ipbl]/ec_tropmeter[1:ipbl]*zpos[1:ipbl])/np.sum(zpos[1:ipbl])*100)
  print('tropics PBL:',np.max(dtropeta[1:ipbl]/ec_tropmeter[1:ipbl])*100,
        np.max(dtropmeter[1:ipbl]/ec_tropmeter[1:ipbl])*100)
  itr=np.abs(zpos-troptr).argmin()
  print('tropics tropo:', np.sum(dtropeta[ipbl:itr]/ec_tropmeter[ipbl:itr]*zpos[ipbl:itr])/np.sum(zpos[ipbl:itr])*100,
        np.sum(dtropmeter[ipbl:itr]/ec_tropmeter[ipbl:itr]*zpos[ipbl:itr])/np.sum(zpos[ipbl:itr])*100)
  print('tropics tropo max:', np.max(dtropeta[ipbl:itr]/ec_tropmeter[ipbl:itr])*100,
        np.max(dtropmeter[ipbl:itr]/ec_tropmeter[ipbl:itr])*100)
  istr=np.abs(zpos-5e4).argmin()
  print('tropics strat:', np.sum(dtropeta[itr:istr]/ec_tropmeter[itr:istr]*zpos[itr:istr])/np.sum(zpos[itr:istr])*100,
        np.sum(dtropmeter[itr:istr]/ec_tropmeter[itr:istr]*zpos[itr:istr])/np.sum(zpos[itr:istr])*100)
  print('tropics strat max:', np.max(dtropeta[itr:istr]/ec_tropmeter[itr:istr])*100,
        np.max(dtropmeter[itr:istr]/ec_tropmeter[itr:istr])*100)
  print('PBL','Tropo','Strato', zpos[ipbl],zpos[itr],zpos[istr])