import numpy as np
import sys
import os
import h5py
import re
# import eccodes
# import cfgrib
import xarray as xr #cite as: Hoyer, S. & Hamman, J., (2017). xarray: N-D labeled Arrays and Datasets in Python. Journal of Open Research Software. 5(1), p.10. DOI: http://doi.org/10.5334/jors.148
import gc
import glob
import copy
#from sklearn.neighbors import BallTree
import matplotlib.pyplot as plt
import netCDF4 as nv
from scipy.interpolate import interp1d,interp2d,griddata
from scipy.interpolate import RectBivariateSpline as rbs
from scipy.interpolate import RegularGridInterpolator as rgi
from scipy.optimize import brentq, curve_fit, minimize_scalar, minimize, Bounds
from scipy.stats import binned_statistic_2d, binned_statistic_dd, binned_statistic
from sklearn.neighbors import BallTree
import pandas as pa
import multiprocessing as mp
from mpl_toolkits.axes_grid1 import make_axes_locatable
import pickle
import cartopy.crs as ccrs
from matplotlib import rcParams

rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = False
rcParams['text.usetex'] = False
rcParams['font.sans-serif'] = ['cmr10', 'Times-Roman']
rcParams['font.weight'] = 'normal'
rcParams['font.size'] = 20
rcParams['xtick.labelsize'] = 20
rcParams['ytick.labelsize'] = 20
rcParams['axes.labelsize'] = 22
rcParams['axes.linewidth'] = 1
rcParams['axes.unicode_minus'] = False
rcParams['xtick.minor.width'] = 1
rcParams['ytick.minor.width'] = 1
rcParams['xtick.major.width'] = 1
rcParams['ytick.major.width'] = 1
rcParams['xtick.minor.size'] = 4
rcParams['ytick.minor.size'] = 4
rcParams['xtick.major.size'] = 5
rcParams['ytick.major.size'] = 5
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'
rcParams['mathtext.fontset'] = 'cm'


import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.colors as colors
from matplotlib import gridspec
from matplotlib import ticker
import matplotlib as mpl
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import NullFormatter
from collections import OrderedDict, Counter
from matplotlib.lines import Line2D
from matplotlib.patches import Patch, Rectangle, Ellipse
import matplotlib.patheffects as pe

"""
------------------------------------------

------------------------------------------

-----------Notes--------------------------------------------------------
"""



class PartInfo():
  """
  Fields in partposit files
  ---------------------------------
  xlon = latitude (degrees)
  ylat = longitude (degrees)
  z = heigth (meters)
  topo = topography
  tro = tropopause
  hmix = PBL height
  rho = density
  qv = specific humidity
  pv = potential vorticity
  tt = temperature
  uu = 
  vv =
  """ 

  def __init__(self, path_to_directory="/home/lucie/SizeTesting/output_1e6_23", 
    partposit_nc_fname = "partposit_average.nc",interpolatetopo=True):

    self.path = path_to_directory
    #Read in the header
    #self.header = read_header_bin(path_to_directory=self.path)
    #Npart from the header is not accurate when domainfilling is used, so this will be set when data is read instead
    self.npart = None

    self.partposit_nc_fname = partposit_nc_fname
    
    #Open netcdf particledata using xarray and naming it self._pd
    self._read_partposit_nc()
    self.coords = None

    self.kdtree = {}

    #2D and 3D fields interpolation (if required)
    self.topography = None
    self.tropopause = None
    self.hmixing = None
    self.interpolatetopo = interpolatetopo

  def _read_partposit_nc(self):
    tempfile = self.path + '/' + self.partposit_nc_fname
    self._pd = xr.open_dataset(tempfile, engine="netcdf4")
    #print("netCDF4 file opened.")

  @property
  def lon(self):
    return self._pd.longitude

  @property
  def lon_av(self):
    return self._pd.longitude_av

  @property
  def lat(self):
    return self._pd.latitude

  @property
  def lat_av(self):
    return self._pd.latitude_av

  @property
  def z(self):
    return self._pd.height

  @property
  def z_av(self):
    return self._pd.height_av

  @property
  def topo(self):
    if self.interpolatetopo:
      if self.topography is None:
        topo_inter = self.interpolate2d(self._pd.topo,0)
        #allocate memory for topography per particle
        self.topography = np.zeros((len(self.lon),len(self.lon[0,:])))
        #For each timestep, interpolate the position of the particle to the grid
        for itime in range(len(self.topography[0,:])):

          topo_inter = self.interpolate2d(self._pd.topo,0)
          self.topography[:,itime]  = topo_inter(self.lat[:,itime],self.lon[:,itime],grid=False)
          mini=np.min(self._pd.topo).values
          maxi=np.max(self._pd.topo).values
          self.topography[self.topography[:,itime]<mini,itime]=mini
          self.topography[self.topography[:,itime]>maxi,itime]=maxi
      return self.topography
    else:
      return self._pd.topo

  @property
  def tro(self):
    if len(self._pd.tr.dims)==3:
      if self.tropopause is None:
        self.tropopause = np.zeros((len(self.xlon),len(self.xlon[0,:])))
        for itime in range(len(self.tropopause[0,:])):
          tro_inter = self.interpolate2d(self._pd.tr,itime)
          self.tropopause[:,itime] = tro_inter(self.ylat[:,itime],self.xlon[:,itime],grid=False)
          mini=np.min(self._pd.tr).values
          maxi=np.max(self._pd.tr).values
          self.tropopause[self.tropopause[:,itime]<mini,itime]=mini
          self.tropopause[self.tropopause[:,itime]>maxi,itime]=maxi
      return self.tropopause
    else:
      return self._pd.tr

  @property
  def hmix(self):
    if len(self._pd.hmix.dims)==3:
      if self.hmixing is None:
        self.hmixing = np.zeros((len(self.xlon),len(self.xlon[0,:])))
        for itime in range(len(self.hmixing[0,:])):
          hmix_inter = self.interpolate2d(self._pd.hmix,itime)
          self.hmixing[:,itime] = hmix_inter(self.ylat[:,itime],self.xlon[:,itime],grid=False)
          mini=np.min(self._pd.hmix).values
          maxi=np.max(self._pd.hmix).values
          self.hmixing[self.hmixing[:,itime]<mini,itime]=mini
          self.hmixing[self.hmixing[:,itime]>maxi,itime]=maxi
      return self.hmixing
    else:
      return self._pd.hmix

  @property
  def rho(self):
    return self._pd.rho

  @property
  def rho_av(self):
    return self._pd.rho_av

  @property
  def qv(self):
    return self._pd.qv

  @property
  def qv_av(self):
    return self._pd.qv_av

  @property
  def prs(self):
    return self._pd.pr

  @property
  def prs_av(self):
    return self._pd.pr_av

  @property
  def pv(self):
    return self._pd.pv

  @property
  def pv_av(self):
    return self._pd.pv_av

  @property
  def tt(self):
    return self._pd.temperature

  @property
  def tt_av(self):
    return self._pd.temperature_av

  @property
  def mass(self):
    return self._pd.mass001

  @property
  def time(self):
    return np.array(self._pd.time)

  @property
  def particle(self):
    return np.array(self._pd.particle)

  @property 
  def day(self):
    return self._pd.time.dt.day.values

  @property 
  def month(self):
    return self._pd.time.dt.month.values

  @property 
  def year(self):
    return self._pd.time.dt.year.values

  @property
  def hour(self):
    return self._pd.time.dt.hour.values

  @property
  def minute(self):
    return self._pd.time.dt.minute.values

  @property
  def second(self):
    return self._pd.time.dt.second.values

  @property
  def period(self):
    return self._pd.attrs[list(self._pd.attrs.keys())[0]]

  def interpolate2d(self,field,itime):
    #lon = field.lon.values-0.5
    lon=np.arange(0.,360.5,0.5)
    #lat = field.lat.values-90.-0.5
    lat=np.arange(-90,90.5,0.5)
    # x,y=np.meshgrid(lon,lat)
    # xy=np.stack([x.ravel(),y.ravel()],-1)
    # info=np.zeros_like(x)
    # if len(field.dims)==3:
    #   infotemp = field[:,:,itime].values
    #   info[1:,1:]=infotemp
    #   info[1:,0]=infotemp[:,1]
    #   info[0,1:]=infotemp[1,:]
    #   info[0,0]=infotemp[1,1]
    # else:
    #   infotemp = field[:,:].values
    #   info[1:,1:]=infotemp
    #   info[1:,0]=infotemp[:,1]
    #   info[0,1:]=infotemp[1,:]
    #   info[0,0]=infotemp[1,1]
    # return griddata((x.ravel(),y.ravel()),info.ravel(),(self.xlon[:,itime],self.ylat[:,itime]))
    if len(field.dims)==2:
      info = field[:,:].values
    else:
      info = field[:,:,itime].values
    return rbs(lat,lon,info)


def volume_calc(grid_xlon, grid_ylat, grid_z,topo=None):
  grid_vol = np.zeros((len(grid_ylat)-1, len(grid_xlon)-1, len(grid_z)-1))
  grid_xrad = np.deg2rad(grid_xlon)
  grid_yrad = np.deg2rad(grid_ylat)+0.5*np.pi
  zfactor = np.ones_like(grid_vol)*1./3*((6371229.+grid_z[1:])**3-(6371229.+grid_z[:-1])**3)

  for i in range(len(grid_xrad)-1):
    for j in range(len(grid_yrad)-1):
      if (topo is not None):
        ll=np.abs(grid_z[:]-topo[j,i]).argmin()
        if grid_z[ll]>topo[j,i]:
          if ll<=0:
            ll=0
          else:
            zfactor[j,i,:ll-1]=0.
            zfactor[j,i,ll-1]=1./3*((6371229.+grid_z[ll+1])**3-(6371229.+topo[j,i])**3)
          # else:
          #   ll=ll-1
        elif grid_z[ll]<=topo[j,i]:
          zfactor[j,i,:ll]=0.
          zfactor[j,i,ll]=1./3*((6371229.+grid_z[ll+1])**3-(6371229.+topo[j,i])**3)

      grid_vol[j, i, :] = (grid_xrad[i+1]-grid_xrad[i])*(np.cos(grid_yrad[j])-np.cos(grid_yrad[j+1]))*zfactor[j,i,:]
  return grid_vol

def convert_time(times):
  return times[0]*1000000+times[1]*10000+times[2]*100+times[3]

def select_files_LARA(path_to_directory='/jetfs/scratch/lbakels/LARA/197901-198503/output/',
  start_time=[1980,1,1,0],end_time=[1980,1,2,0]):
  
  pd = {}

  files = os.listdir(path_to_directory)
  
  s_time=convert_time(start_time)
  e_time=convert_time(end_time)

  fname='partoutput'
  ff = []
  times= []

  for file in files:
    if file.endswith(".nc") and file.startswith(fname):
      ff.append(file)
      times.append(int(re.findall(r'\d{14}',file)[0][0:10]))
  #times = [int(filename[11:25]) for filename in ff]
  times = np.array(times)
  order = np.argsort(times)
  ff = list(np.array(ff)[order])
  times = times[order]

  nf=0
  pd_times = {}
  i=-1
  for file in ff:
    i+=1
    if (e_time<times[i]):
      break
    if (i<len(ff)-1) and (s_time>times[i+1]):
      continue
    #print('Open',path_to_directory+'/'+file)
    pd_temp = PartInfo(path_to_directory=path_to_directory,partposit_nc_fname=file,interpolatetopo=True)
    pd_temp_times = pd_temp.year*1000000+pd_temp.month*10000+pd_temp.day*100+pd_temp.hour
    if len(pd_temp_times)==0:
      continue
    if ( s_time <= pd_temp_times[-1] and e_time >= pd_temp_times[0] ):
      pd[file] = pd_temp
      pd_times[file] = pd_temp_times
      nf += 1
    elif (pd_temp_times[0])>e_time:
      break
  if nf==0:
    print('No files available within the requested time period')
    return

  return pd,pd_times

def compute_masked_arrays(partdata,ecdata,lat1,lat2,ndata,gridvol,tropo,hmix,topo,topogrid):
  '''
  A:
  1) Only consider gridcells above topography
  2) Take the difference between the particle computed densities and ECMWF densities per grid cell
  3) Take the average of all gridcells per height level
  B:
  1) Only consider gridcells above topography
  2) Take the volume weighted average of the particle computed densities of all gridcells per height level
  C:
  1) Only consider gridcells above topography
  2) Only consider gridcells that have particles inside for taking the average density
  3) Take the volume weighted average of the mean ECMWF density of all gridcells per height level
  '''
  # masked1 = np.ma.masked_where(np.isnan(partdata[lat1:lat2,:,:]) | (np.isinf(partdata[lat1:lat2,:,:])) |
  #   (ndata[lat1:lat2,:,:]<3) ,np.abs(partdata[lat1:lat2,:,:]-ecdata[lat1:lat2,:,:]))
  # darray=masked1.mean(axis=(0,1))
  mintop=1e10
  topotemp = np.transpose(np.transpose(np.ones_like(partdata),(2,0,1))*topogrid,(1,2,0))

  masked1 = np.ma.masked_where(np.isnan(partdata[lat1:lat2,:,:]) | (np.isinf(partdata[lat1:lat2,:,:])) | (topotemp[lat1:lat2,:,:]>mintop)
    ,partdata[lat1:lat2,:,:]*gridvol[lat1:lat2,:,:])
  masked2 = np.ma.masked_where(np.isnan(partdata[lat1:lat2,:,:]) | (np.isinf(partdata[lat1:lat2,:,:])) | (topotemp[lat1:lat2,:,:]>mintop)
    ,gridvol[lat1:lat2,:,:])
  part_array=masked1.sum(axis=(0,1))/masked2.sum(axis=(0,1))

  #mintop=100
  masked1 = np.ma.masked_where(#np.isnan(partdata[lat1:lat2,:,:]) | (np.isinf(partdata[lat1:lat2,:,:])) |
    (ecdata[lat1:lat2,:,:]==0) | (topotemp[lat1:lat2,:,:]>mintop)
    ,ecdata[lat1:lat2,:,:]*gridvol[lat1:lat2,:,:])
  masked2 = np.ma.masked_where(#np.isnan(partdata[lat1:lat2,:,:]) | (np.isinf(partdata[lat1:lat2,:,:])) |
    (ecdata[lat1:lat2,:,:]==0) | (topotemp[lat1:lat2,:,:]>mintop)
    ,gridvol[lat1:lat2,:,:])

  ec_array=masked1.sum(axis=(0,1))/masked2.sum(axis=(0,1))

  #Compute hmix and tropopause
  gridvolcol=np.sum(gridvol,axis=(1,2))
  masked1 = np.ma.masked_where((tropo[lat1:lat2]==0) | (topo[lat1:lat2]>mintop),tropo[lat1:lat2]*gridvolcol[lat1:lat2])
  masked2 = np.ma.masked_where((tropo[lat1:lat2]==0) | (topo[lat1:lat2]>mintop),gridvolcol[lat1:lat2])

  avtr=masked1.sum()/masked2.sum()

  masked1 = np.ma.masked_where((hmix[lat1:lat2]==0) | (topo[lat1:lat2]>mintop),hmix[lat1:lat2]*gridvolcol[lat1:lat2])
  masked2 = np.ma.masked_where((hmix[lat1:lat2]==0) | (topo[lat1:lat2]>mintop),gridvolcol[lat1:lat2])

  avpbl=masked1.sum()/masked2.sum()

  masked1 = np.ma.masked_where((topo[lat1:lat2]==0) | (topo[lat1:lat2]>mintop),topo[lat1:lat2]*gridvolcol[lat1:lat2])
  masked2 = np.ma.masked_where((topo[lat1:lat2]==0) | (topo[lat1:lat2]>mintop),gridvolcol[lat1:lat2])

  avtopo=masked1.sum()/masked2.sum()

  return avtr,avpbl,part_array,ec_array,avtopo

def plot_density_test( path_to_eta, path_to_meter, z=[0,1e5],):

  pdeta,pdeta_times = select_files_LARA(
    path_to_directory=path_to_eta,
    start_time=[1970,6,1,1],
    end_time=[1970,6,30,23])

  pdmeter,pdmeter_times = select_files_LARA(
    path_to_directory=path_to_meter,
    start_time=[1970,6,1,1],
    end_time=[1970,6,30,23])

  #grid from sea level
  # To get the correct value on the edges of the bins, we need to
  # substract 0.25 of the bins! The edges of the grid_vol are zero and are excluded from computations.
  grid_ylat=np.arange(-90.25,90.26,0.5)
  grid_xlon=np.arange(-0.25,360,0.5)
  grid_z = np.concatenate([np.array([0.,25.,50.,75.]),np.logspace(2,np.log10(z[1]),100)])

  #zpos = np.logspace(np.log10(grid_z[0]+(grid_z[1]-grid_z[0])/2.),np.log10(grid_z[-2]+(grid_z[-1]-grid_z[-2])/2.),99)
  zpos=grid_z[:-1]+(grid_z[1:]-grid_z[:-1])/2.
  
  hmix=None
  itot=0
  for ipd in pdeta.keys():
    if hmix is None:
      hmix = np.mean(pdeta[ipd]._pd.hmix.values,axis=(1,2)) # double value at first and final lon (not a problem)
      tropo = np.mean(pdeta[ipd]._pd.tr.values,axis=(1,2))
      #Mass per particle
      numpart = len(np.where(pdeta[ipd]._pd['lon'][:,0]>0)[0])
      massparteta = 5.09256513E18/numpart #len(pdeta[ipd].particle)5913582
      masspartmeter = 5.09256513E18/numpart #len(pdmeter[ipd].particle)
    else:
      hmix = (hmix + np.mean(pdeta[ipd]._pd.hmix.values,axis=(1,2)))
      tropo = (tropo + np.mean(pdeta[ipd]._pd.tr.values,axis=(1,2)))
    itot=itot+1

  hmix=hmix/itot
  tropo=tropo/itot
  topo = pdeta[ipd]._pd.topo.values
  topo2=np.mean(topo,axis=(1))

  grid_vol=volume_calc(grid_xlon,grid_ylat,grid_z,topo)

  #Sum up particles within range
  aantal=0
  for ipd in pdeta.keys():
    pdeta_end=pdeta[ipd]
    pdmeter_end=pdmeter[ipd]
    length=len(pdeta_end.hour)
    print(ipd,length)
    for i in range(length):
      topoeta = pdeta_end.topo[:,i]
      topometer = pdmeter_end.topo[:,i]
      lon1eta = pdeta_end.lon[:,i].values
      #lon1eta = np.where(lon1eta>180.,lon1eta-360.,lon1eta)
      lat1eta = pdeta_end.lat[:,i].values
      z1eta = pdeta_end.z[:,i].values+topoeta
      lon1meter = pdmeter_end.lon[:,i].values
      #lon1meter = np.where(lon1meter>180.,lon1meter-360.,lon1meter)
      lat1meter = pdmeter_end.lat[:,i].values
      z1meter = pdmeter_end.z[:,i].values+topometer

      #Bin the results according to specified grid
      grid_n1eta_tmp,edges = np.histogramdd([lat1eta,lon1eta,z1eta], 
        bins=[grid_ylat,grid_xlon,grid_z])
      grid_n1meter_tmp,edges = np.histogramdd([lat1meter,lon1meter,z1meter], 
        bins=[grid_ylat,grid_xlon,grid_z])

      if aantal==0:
        grid_n1eta=np.zeros_like(grid_vol)
        grid_n1meter=np.zeros_like(grid_vol)
        grid_n1eta_topo=np.zeros_like(grid_vol)
        grid_n1meter_topo=np.zeros_like(grid_vol)

      grid_n1eta=grid_n1eta+grid_n1eta_tmp
      grid_n1meter=grid_n1meter+grid_n1meter_tmp

      aantal=aantal+1

    pdeta_end._pd.close()
    pdmeter_end._pd.close()

  #From particle distribution
  #NaN below topography
  rho_n1eta = grid_n1eta/grid_vol*massparteta/aantal
  rho_n1meter = grid_n1meter/grid_vol*masspartmeter/aantal


  #Average difference between era5 and particle distribution
  def ew(td):
    y=373.16/td
    a=-7.90298*(y-1.)
    a=a+(5.02808*0.43429*np.log(y))
    c=(1.-(1./y))*11.344
    c=-1.+(10.**c)
    c=-1.3816*c/(10.**7)
    d=(1.-y)*3.49149
    d=-1.+(10.**d)
    d=8.1328*d/(10.**3)
    y=a+c+d
    return 101324.6*(10.**y)

  # Loading ERA-5 data
  ERA5basepath="/jetfs/shared-data/ECMWF/ERA5_glob_0.5deg_1h/1970/06/"
  EAfiles = os.listdir(ERA5basepath)
  # Computing model pressure levels
  df = pa.read_csv('table.csv')
  akm = df['a [Pa]'].values
  bkm = df['b'].values

  akz=0.5*(akm[1:]+akm[:-1])
  bkz=0.5*(bkm[1:]+bkm[:-1])
  i=0
  for EAfile in EAfiles:
    qv = xr.open_dataset(ERA5basepath+EAfile,
      engine="cfgrib",backend_kwargs={'filter_by_keys':{'shortName':'q'},'indexpath':''})['q'].values
    tth = xr.open_dataset(ERA5basepath+EAfile,
      engine="cfgrib",backend_kwargs={'filter_by_keys':{'shortName':'t'},'indexpath':''})['t'].values
    sp = xr.open_dataset(ERA5basepath+EAfile,
      engine="cfgrib",backend_kwargs={'filter_by_keys':{'shortName':'sp'},'indexpath':''})['sp'].values
    tt2 = xr.open_dataset(ERA5basepath+EAfile,
      engine="cfgrib",backend_kwargs={'filter_by_keys':{'shortName':'2t'},'indexpath':''})['t2m'].values
    td2 = xr.open_dataset(ERA5basepath+EAfile,
      engine="cfgrib",backend_kwargs={'filter_by_keys':{'shortName':'2d'},'indexpath':''})['d2m'].values

    # z = xr.open_dataset(ERA5basepath+EAfile,
    #   engine="cfgrib",backend_kwargs={'filter_by_keys':{'shortName':'z'},'indexpath':''})['z'].values


    prs = np.zeros((len(akz),len(sp),len(sp[0])))
    for iz in range(len(akz)):
      prs[iz,:,:] = akz[iz]+bkz[iz]*sp

    #Flip z axis
    prs=np.flip(prs,axis=0)
    tth=np.flip(tth,axis=0)
    qv=np.flip(qv,axis=0)

    r_air=287.05
    ga=9.81
    const=r_air/ga
    tv=tth*(1.+0.608*qv)
    tv[0,:,:]=tt2*(1.+0.378*ew(td2)/sp)

    #Density
    rhotmp = prs/(r_air*tv)

    heightstmp=np.zeros_like(tv)
    heightstmp[0,:,:] = np.roll(np.flip(topo[:,:-1],axis=0),360,axis=1)
    for iz in range(1,len(heightstmp)):
      heightstmp[iz,:,:] = np.where(abs(tv[iz,:,:]-tv[iz-1])>0.2,
        heightstmp[iz-1,:,:] + const*np.log(prs[iz-1,:,:]/prs[iz,:,:]) * 
        (tv[iz,:,:]-tv[iz-1,:,:])/np.log(tv[iz,:,:]/tv[iz-1,:,:]),
        heightstmp[iz-1,:,:] + const*np.log(prs[iz-1,:,:]/prs[iz,:,:])*tv[iz,:,:])

    if i==0:
      heights = np.roll(np.flip(np.transpose(heightstmp,(1,2,0)),axis=0),360,axis=1)
      rho = np.roll(np.flip(np.transpose(rhotmp,(1,2,0)),axis=0),360,axis=1)
    else:
      heights = heights + np.roll(np.flip(np.transpose(heightstmp,(1,2,0)),axis=0),360,axis=1)
      rho = rho + np.roll(np.flip(np.transpose(rhotmp,(1,2,0)),axis=0),360,axis=1)
    i=i+1
    del qv
    del tth
    del sp
    del tt2
    del td2

  heights=heights/i
  rho=rho/i

  #Interpolating densities to fit the zpos levels
  rho2 = np.zeros_like(grid_vol)
  for jy in range(len(grid_ylat)-1):
    for ix in range(len(grid_xlon)-1):
      inter_tmp = interp1d(heights[jy,ix,:],rho[jy,ix,:])
      lmin = np.abs(zpos-heights[jy,ix,0]).argmin()
      lmax = np.abs(zpos-heights[jy,ix,-1]).argmin()-1
      if heights[jy,ix,lmin]>zpos[lmin]:
        lmin=lmin+1
      if heights[jy,ix,lmax]<zpos[lmax]:
        lmax=lmax-1
      rho2[jy,ix,lmin:lmax] = inter_tmp(zpos[lmin:lmax])

  # #Poles
  polestr,polespbl,poleseta,ec_poleseta,polestopo=compute_masked_arrays(
    np.concatenate((rho_n1eta[0:48,:,:],rho_n1eta[312:361,:,:]),axis=0),
    np.concatenate((rho2[0:48,:,:],rho2[312:361,:,:]),axis=0),
    0,-1,
    np.concatenate((grid_n1eta[0:48,:,:],grid_n1eta[312:361,:,:]),axis=0),
    np.concatenate((grid_vol[0:48,:,:],grid_vol[312:361,:,:]),axis=0),
    np.concatenate((tropo[0:48],tropo[312:361]),axis=0),
    np.concatenate((hmix[0:48],hmix[312:361]),axis=0),
    np.concatenate((topo2[0:48],topo2[312:361]),axis=0),
    np.concatenate((topo[0:48,:-1],topo[312:361,:-1]),axis=0)
    )
  dpoleseta=np.abs(poleseta-ec_poleseta)
  
  polestr,polespbl,polesmeter,ec_polesmeter,polestopo=compute_masked_arrays(
    np.concatenate((rho_n1meter[0:48,:,:],rho_n1meter[312:361,:,:]),axis=0),
    np.concatenate((rho2[0:48,:,:],rho2[312:361,:,:]),axis=0),
    0,-1,
    np.concatenate((grid_n1meter[0:48,:,:],grid_n1meter[312:361,:,:]),axis=0),
    np.concatenate((grid_vol[0:48,:,:],grid_vol[312:361,:,:]),axis=0),
    np.concatenate((tropo[0:48],tropo[312:361]),axis=0),
    np.concatenate((hmix[0:48],hmix[312:361]),axis=0),
    np.concatenate((topo2[0:48],topo2[312:361]),axis=0),
    np.concatenate((topo[0:48,:-1],topo[312:361,:-1]),axis=0)
    )
  dpolesmeter=np.abs(polesmeter-ec_polesmeter)

  datafile = open('poles.pickle','wb')
  pole_pickle = {}
  pole_pickle['polestr']=polestr
  pole_pickle['polespbl']=polespbl
  pole_pickle['poleseta']=poleseta
  pole_pickle['ec_poleseta']=ec_poleseta
  pole_pickle['polestopo']=polestopo
  pole_pickle['polesmeter']=polesmeter
  pole_pickle['ec_polesmeter']=ec_polesmeter
  pickle.dump(pole_pickle,datafile)
  datafile.close()

  midlattr,midlatpbl,tmp1,ec_tmp1,midlattopo=compute_masked_arrays(rho_n1eta,rho2,48,134,grid_n1eta,grid_vol,tropo,hmix,topo2,topo[:,:-1])
  midlattr,midlatpbl,tmp2,ec_tmp2,midlattopo=compute_masked_arrays(rho_n1eta,rho2,226,312,grid_n1eta,grid_vol,tropo,hmix,topo2,topo[:,:-1])
  #dmidlateta=(dtmp1+dtmp2)/2.
  midlateta=(tmp1+tmp2)/2.
  ec_midlateta=(ec_tmp1+ec_tmp2)/2.
  dmidlateta=np.abs(midlateta-ec_midlateta)

  midlattrm,midlatpblm,tmp1,ec_tmp1,midlattopo=compute_masked_arrays(rho_n1meter,rho2,48,134,grid_n1meter,grid_vol,tropo,hmix,topo2,topo[:,:-1])
  midlattrm,midlatpblm,tmp2,ec_tmp2,midlattopo=compute_masked_arrays(rho_n1meter,rho2,226,312,grid_n1meter,grid_vol,tropo,hmix,topo2,topo[:,:-1])
  #dmidlatmeter=(dtmp1+dtmp2)/2.
  midlatmeter=(tmp1+tmp2)/2.
  ec_midlatmeter=(ec_tmp1+ec_tmp2)/2.
  dmidlatmeter=np.abs(midlatmeter-ec_midlatmeter)

  datafile = open('midlat.pickle','wb')
  ml_pickle = {}
  ml_pickle['midlattr']=midlattr
  ml_pickle['midlatpbl']=midlatpbl
  ml_pickle['midlateta']=midlateta
  ml_pickle['ec_midlateta']=ec_midlateta
  ml_pickle['midlatmeter']=midlatmeter
  ml_pickle['ec_midlatmeter']=ec_midlatmeter
  ml_pickle['midlattopo']=midlattopo
  pickle.dump(ml_pickle,datafile)
  datafile.close()


  #Tropics
  troptr,troppbl,tropeta,ec_tropeta,troptopo=compute_masked_arrays(rho_n1eta,rho2,134,226,grid_n1eta,grid_vol,tropo,hmix,topo2,topo[:,:-1])
  troptrm,troppblm,tropmeter,ec_tropmeter,troptopo=compute_masked_arrays(rho_n1meter,rho2,134,226,
    grid_n1meter,grid_vol,tropo,hmix,topo2,topo[:,:-1])
  dtropeta=np.abs(tropeta-ec_tropeta)
  dtropmeter=np.abs(tropmeter-ec_tropmeter)

  datafile = open('topics.pickle','wb')
  tr_pickle = {}
  tr_pickle['troptr']=troptr
  tr_pickle['troppbl']=troppbl
  tr_pickle['tropeta']=tropeta
  tr_pickle['ec_tropeta']=ec_tropeta
  tr_pickle['tropmeter']=tropmeter
  tr_pickle['ec_tropmeter']=ec_tropmeter
  tr_pickle['troptopo']=troptopo
  pickle.dump(tr_pickle,datafile)
  datafile.close()
