import numpy as np
import matplotlib.pyplot as plt
from netCDF4 import Dataset

# Constants
xet1_2 = 5.25  # days
lamda = np.log(2) / xet1_2

# Station coordinates
stationslatlon = np.array([
    [21.5, -158.0], [62.5, -114.5], [37.2, -99.8], [38.0, -78.0], [47.6, -52.7],
    [78.2, 15.4], [59.4, 17.9], [47.9, 7.9], [47.9, 106.3], [44.2, 132.0], [19.3, 166.6]
])
name_stations = ['Oahu', 'Yellowknife', 'Ashland', 'Charlottesville', 'St. John',
                 'Spitsbergen', 'Stockholm', 'Schauinsland', 'Ulaanbaatar', 'Ussuriysk', 'Wake Island']

stationslatlonround = np.round(stationslatlon)
stationslatlonindex = np.round(stationslatlon / 2) + np.array([45, 90])

# Load NetCDF file
fn = r"E:\run_fuku\new_runs\nogf_ratio_00062_uni\output\grid_conc_20110310150000.nc"
filenameout = r"E:\run_fuku\new_runs\processed_test\cs137_xcol_nogf_ratio00062_uni.dat"

nc = Dataset(fn, 'r')

# Read variables
LAT = np.array(nc.variables['latitude'][:])
LON = np.array(nc.variables['longitude'][:])
LEV = np.array(nc.variables['height'][:])
QFLX = np.array(nc.variables['spec001_mr'][:])

timesel = 84
sumtime = 3
sumvert = 4
nlayers = 6

# Process QFLX
QFLXplanetime = QFLX[:, :, :sumvert, timesel:timesel + sumtime].sum(axis=-1) / sumtime
multiply = np.ones_like(QFLXplanetime)
multiply[:, :, 2:4] += 1
QFLXplanetime *= multiply
QFLXplanetime = QFLXplanetime.sum(axis=2) / nlayers

# BlockMean equivalent function
def BlockMean(data, block_x, block_y):
    shape = (data.shape[0] // block_x, block_x, data.shape[1] // block_y, block_y)
    return data.reshape(shape).mean(axis=(1, 3))

# Apply BlockMean
QFLXplanetimer = BlockMean(QFLXplanetime, 2, 2)
LONr = BlockMean(LON, 2, 1)
LATr = BlockMean(LAT, 2, 1)

# Further process QFLXplanetimer
QFLXplanetime70p = QFLXplanetimer[:, 70:90]
QFLXplanetime70pr = BlockMean(QFLXplanetime70p, 2, 1)
QFLXplanetimer[::2, 70:90] = QFLXplanetime70pr
QFLXplanetimer[1::2, 70:90] = QFLXplanetime70pr

QFLXplanetime70m = QFLXplanetimer[:, :20]
QFLXplanetime70mr = BlockMean(QFLXplanetime70m, 2, 1)
QFLXplanetimer[::2, :20] = QFLXplanetime70mr
QFLXplanetimer[1::2, :20] = QFLXplanetime70mr

# Plotting
plt.figure(36)

# Custom colormap loading (this should be adjusted to Python)
# Replace the colormap loading with actual data or use an equivalent colormap
colormap = plt.get_cmap('viridis')

mcmax = np.max(QFLXplanetimer)
mcmin = max(np.min(QFLXplanetimer), mcmax / 1e6)
edges = np.linspace(np.log10(mcmin), np.log10(mcmax), 101)

plt.contourf(LONr, LATr, np.log10(QFLXplanetimer.T), edges, cmap=colormap)
plt.colorbar(orientation='horizontal')
plt.xlabel('Longitude')
plt.ylabel('Latitude')
plt.grid(True, linestyle=':', linewidth=0.5)
plt.xlim([-180, 180])
plt.ylim([-90, 90])

# Plot station locations
QFLXplanetimer_trasposta = QFLXplanetimer.T
concstation = []
for is_ in range(11):
    plt.plot(stationslatlonround[is_, 1], stationslatlonround[is_, 0], 'ro', markersize=15)
    concstation.append(QFLXplanetimer_trasposta[stationslatlonindex[is_, 0], stationslatlonindex[is_, 1]])

plt.show()

# Time series extraction
stationslatlonindex360 = np.zeros((4, 11, 2), dtype=int)
stationslatlonindex360[0, :, :] = np.round(stationslatlon + np.array([90, 180]))
stationslatlonindex360[1, :, :] = np.round(stationslatlon + np.array([91, 180]))
stationslatlonindex360[2, :, :] = np.round(stationslatlon + np.array([90, 181]))
stationslatlonindex360[3, :, :] = np.round(stationslatlon + np.array([91, 181]))

QFLXpoint_intime_day = np.zeros((4, 11, 92))
time_inday = np.arange(1, 93)

for is_ in range(11):
    for iLL in range(4):
        QFLXpoint_intime1 = QFLX[stationslatlonindex360[iLL, is_, 1], stationslatlonindex360[iLL, is_, 0], :2, :]
        QFLXpoint_intime2 = QFLX[stationslatlonindex360[iLL, is_, 1], stationslatlonindex360[iLL, is_, 0], 2:sumvert, :]
        QFLXpoint_intime2 *= 2
        QFLXpoint_intime = QFLXpoint_intime1 + QFLXpoint_intime2
        QFLXpoint_intime_s = QFLXpoint_intime.sum(axis=0)
        QFLXpoint_intime_a = QFLXpoint_intime_s / nlayers
        QFLXpoint_intime_day[iLL, is_, :] = BlockMean(QFLXpoint_intime_a, 1, sumtime)

QFLXpoint_intime_day_sa = QFLXpoint_intime_day.sum(axis=0) / 4

# Plot time series
plt.figure(43)
for is_ in range(11):
    plt.semilogy(time_inday, QFLXpoint_intime_day_sa[is_, :])
plt.legend(name_stations)
plt.show()

# Save data to file
fmt = "%12.5E " * (92 - 1) + "%12.5E\n"
with open(filenameout, 'w') as frel:
    for is_ in range(11):
        frel.write(fmt % tuple(QFLXpoint_intime_day_sa[is_, :]))

QFLXplanetime1 = QFLXplanetime
LON1 = LON
