import numpy as np
import matplotlib.pyplot as plt

from matplotlib import rcParams, ticker
from matplotlib.lines import Line2D

rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = False
rcParams['text.usetex'] = False
rcParams['font.sans-serif'] = ['cmr10', 'Times-Roman']
rcParams['font.weight'] = 'normal'
rcParams['font.size'] = 28
rcParams['xtick.labelsize'] = 28
rcParams['ytick.labelsize'] = 28
rcParams['axes.labelsize'] = 28
rcParams['axes.linewidth'] = 1
rcParams['axes.unicode_minus'] = False
rcParams['xtick.minor.width'] = 1
rcParams['ytick.minor.width'] = 1
rcParams['xtick.major.width'] = 1
rcParams['ytick.major.width'] = 1
rcParams['xtick.minor.size'] = 4
rcParams['ytick.minor.size'] = 4
rcParams['xtick.major.size'] = 5
rcParams['ytick.major.size'] = 5
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'
rcParams['mathtext.fontset'] = 'cm'

from matplotlib.ticker import LogLocator

# Define the station names and time in days
name_stations = ['Oahu', 'Yellowknife', 'Ashland', 'Charlottesville', 'St. John', 
                 'Spitsbergen', 'Stockholm', 'Schauinsland', 'Ulaanbaatar', 'Ussuriysk', 'Wake Island','Sum']
time_inday = np.arange(1, 93)


# File paths (adjusted to Python format)
filename = (["processed_test/xe133_xcol_nogf_ratio001_uni.dat", "processed_test/xe133_xcol_nogf_ratio00062_uni.dat",
    "processed_test/xe133_xcol_nogf_ratio0005_uni.dat"])
filename2 = (["processed_test/cs137_xcol_nogf_ratio001_uni.dat","processed_test/cs137_xcol_nogf_ratio00062_uni.dat",
    "processed_test/cs137_xcol_nogf_ratio0005_uni.dat"])


fig, ax = plt.subplots(1,3, figsize=(22,10),sharex=True,sharey=True)


colour=plt.cm.Set3

colour= (['tomato','gold','grey','peru','lawngreen',
    'mediumseagreen','darkturquoise','dodgerblue','lightsteelblue','mediumpurple','violet'])

legendboxes = np.array([Line2D([0],[0],marker='o',markersize=8,
            markerfacecolor='none',markeredgecolor=colour[i],linestyle='') for i in range(11)])
legendboxes = np.append(legendboxes,Line2D([0],[0], marker='s', linewidth=2.0, markersize=10,
                 color=[1, 0.6, 0.6],markeredgecolor='red', markerfacecolor=[1, 0.6, 0.6]))

legend1 = plt.legend(legendboxes,name_stations, loc='lower left',ncol=2,fontsize=21)
titles = ['r$_{\\rm icl}$ = 0.01', 'r$_{\\rm icl}$ = 0.0062', 'r$_{\\rm icl}$ = 0.005']
tags=np.array(['(a)', '(b)', '(c)'])
for iax in range(3):
    # Read data from files (assuming importfilexe133dc and importfilecs137 equivalent Python functions are defined)
    xe133dc = np.loadtxt(filename[iax])  # Replace with the actual file reading code
    cs137 = np.loadtxt(filename2[iax])     # Replace with the actual file reading code

    # Summing the data across columns
    xe133dc_sum = np.sum(xe133dc, axis=0)
    cs137_sum = np.sum(cs137, axis=0)

    # Calculating ratios
    ratiocs137suxe133dc = cs137 / xe133dc
    ratiocs137suxe133dc_sum = cs137_sum / xe133dc_sum

    # Calculation of time constants
    t15_65 = -(50. / np.log(ratiocs137suxe133dc[:, 64] / ratiocs137suxe133dc[:, 14]))
    t25_45 = -(20. / np.log(ratiocs137suxe133dc[:, 44] / ratiocs137suxe133dc[:, 24]))
    t45_65 = -(20. / np.log(ratiocs137suxe133dc[:, 64] / ratiocs137suxe133dc[:, 44]))
    t15_90 = -(75. / np.log(ratiocs137suxe133dc[:, 89] / ratiocs137suxe133dc[:, 14]))

    t15_65_sum = -(50. / np.log(ratiocs137suxe133dc_sum[64] / ratiocs137suxe133dc_sum[14]))
    t25_45_sum = -(20. / np.log(ratiocs137suxe133dc_sum[44] / ratiocs137suxe133dc_sum[24]))
    t45_65_sum = -(20. / np.log(ratiocs137suxe133dc_sum[64] / ratiocs137suxe133dc_sum[44]))
    t15_90_sum = -(75. / np.log(ratiocs137suxe133dc_sum[89] / ratiocs137suxe133dc_sum[14]))

    # Plotting
    xdata = time_inday[14:90]  # decay time 15-90 days
    ydata = ratiocs137suxe133dc_sum[14:90]
    ydata2 = np.log(ydata)

    # Polynomial fitting and exponential
    pol = np.polyfit(xdata, ydata2, 1)
    res = np.exp(pol[1]) * np.exp(xdata * pol[0])
    timescale4 = round(-1 / pol[0], 1)
    const = np.exp(pol[1])


    ax[iax].semilogy(xdata, res, '--k', linewidth=2.0,zorder=9999999)

    ax[iax].semilogy(time_inday[:90], ratiocs137suxe133dc_sum[:90], '-s', linewidth=2.0, markersize=10,
                 color=[1, 0.6, 0.6],markeredgecolor='red', markerfacecolor=[1, 0.6, 0.6],zorder=99999)
    for i in range(11):
        ax[iax].semilogy(time_inday[:90], ratiocs137suxe133dc[i, :90], marker='o', markersize=8,
            markerfacecolor='none',markeredgecolor=colour[i],linestyle='')

    legend_labels = np.array([(f'C * exp(-t/{timescale4})')])


    ax[iax].legend(legend_labels, frameon=False,loc='upper right')
    ax[iax].set_xlim([0, 100])
    ax[iax].set_ylim([1e-12, 0.1])

    ax[iax].set_xlabel(r'time [days]', color='k')
    ax[iax].set_title(titles[iax])


    ax[iax].text(0.03,.97,tags[iax],horizontalalignment='left',verticalalignment='top',
        transform=ax[iax].transAxes,color='k',
        bbox=dict(facecolor='white',edgecolor='none',alpha=0.7,boxstyle='round',pad=0.01))
ax[0].set_yticks(ticks=np.logspace(-12,0,7)[:-1])


ax[iax].add_artist(legend1)


a = np.zeros(0)
for i in np.logspace(-12,0,13)[:-1]:
    a=np.append(a,np.arange(1,10)*i)
ax[0].yaxis.set_minor_locator(ticker.FixedLocator(a))
ax[0].yaxis.set_minor_formatter(ticker.NullFormatter())



#ax[2].yaxis.set_minor_locator(LogLocator(base=10, subs=range(-10),numticks=7))
ax[0].set_ylabel(r'Cs137/Xe133$_{decay-corrected}$', color='k')
plt.subplots_adjust(top=0.94, bottom=0.1, left=0.07, right=0.98, wspace=0.1, hspace=0)

plt.savefig('f05.png')
plt.savefig('f05.pdf')
plt.close()

print(timescale4)
