import numpy as np
import sys
import os
import h5py
import re
# import eccodes
# import cfgrib
import xarray as xr #cite as: Hoyer, S. & Hamman, J., (2017). xarray: N-D labeled Arrays and Datasets in Python. Journal of Open Research Software. 5(1), p.10. DOI: http://doi.org/10.5334/jors.148
import gc
import glob
import copy
#from sklearn.neighbors import BallTree
import matplotlib.pyplot as plt
import netCDF4 as nv
from scipy.interpolate import interp1d,interp2d,griddata
from scipy.interpolate import RectBivariateSpline as rbs
from scipy.interpolate import RegularGridInterpolator as rgi
from scipy.optimize import brentq, curve_fit, minimize_scalar, minimize, Bounds
from scipy.stats import binned_statistic_2d, binned_statistic_dd, binned_statistic
from sklearn.neighbors import BallTree
import pandas as pa
import multiprocessing as mp
from mpl_toolkits.axes_grid1 import make_axes_locatable

import cartopy.crs as ccrs
from matplotlib import rcParams

rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = False
rcParams['text.usetex'] = False
rcParams['font.sans-serif'] = ['cmr10', 'Times-Roman']
rcParams['font.weight'] = 'normal'
rcParams['font.size'] = 20
rcParams['xtick.labelsize'] = 20
rcParams['ytick.labelsize'] = 20
rcParams['axes.labelsize'] = 22
rcParams['axes.linewidth'] = 1
rcParams['axes.unicode_minus'] = False
rcParams['xtick.minor.width'] = 1
rcParams['ytick.minor.width'] = 1
rcParams['xtick.major.width'] = 1
rcParams['ytick.major.width'] = 1
rcParams['xtick.minor.size'] = 4
rcParams['ytick.minor.size'] = 4
rcParams['xtick.major.size'] = 5
rcParams['ytick.major.size'] = 5
rcParams['xtick.direction'] = 'in'
rcParams['ytick.direction'] = 'in'
rcParams['mathtext.fontset'] = 'cm'


import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.colors as colors
from matplotlib import gridspec
from matplotlib import ticker
import matplotlib as mpl
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.ticker import NullFormatter
from collections import OrderedDict, Counter
from matplotlib.lines import Line2D
from matplotlib.patches import Patch, Rectangle, Ellipse
import matplotlib.patheffects as pe


def plot_Figure8():

  # ETA
  Aompmemvsceta={}
  Aompmemvsceta['1e6']=np.array([10.7,10.8,10.6,10.6,10.7,10.8,10.9,11.5])
  Aompmemvsceta['1e7']=np.array([14.7,13.6,14.7,14.7,13.1,14.9,15.1,15.5])
  Aompmemvsceta['1e8']=np.array([52.2,52.3,52.2,52.3,52.3,52.3,52.6,52.9])

  #ETA BINDING CORES

  Aompvsceta={}
  Aompvsceta['1e6']={}
  Aompvsceta['1e6']['tps']=np.array([93.9882812,48.7187500,26.2949219,14.8398438,10.1855469,6.68750000,5.32421875,4.75976562,4.82226562])
  Aompvsceta['1e6']['io']=np.array([3.34218740,2.40000010,2.23281240,2.01250005,1.82187498,1.71249998,1.64687502,1.74218750,2.07812500])
  Aompvsceta['1e6']['init']=np.array([27.7304688,15.3750000,9.25000000,6.39062500,5.21875000,3.98437500,3.47656250,3.37500000,3.84765625])
  Aompvsceta['1e7']={}
  Aompvsceta['1e7']['tps']=np.array([271.343750,146.156250,81.4960938,49.0800781,37.7968750,29.9082031,26.0722656,26.5917969,24.4687500])
  Aompvsceta['1e7']['io']=np.array([16.7500000,13.2749996,11.7578125,10.6906252,10.0562496,9.81250000,9.48281288,9.75312519,9.85000038])
  Aompvsceta['1e7']['init']=np.array([55.1250000,31.8750000,20.2421875,14.9375000,13.2812500,12.0625000,10.2109375,10.5312500,10.2500000])
  Aompvsceta['1e8']={}
  Aompvsceta['1e8']['tps']=np.array([1964.59961,1072.21289,605.416016,378.156250,307.486328,247.683594,232.871094,243.851562,213.968750])
  Aompvsceta['1e8']['io']=np.array([143.371872,117.901566,100.615623,93.2640610,89.8312531,86.7890625,85.6921844,87.5578156,81.3000031])
  Aompvsceta['1e8']['init']=np.array([301.625000,184.328125,113.312500,82.2187500,76.1406250,67.5156250,64.1406250,65.5234375,67.0937500])

  Aompjeteta={}
  Aompjeteta['1e6']={}
  Aompjeteta['1e6']['tps']=np.array([127.875000,65.3281250,37.4375000,22.0234375,13.9140625,12.1328125,9.50000000])
  Aompjeteta['1e6']['io']=np.array([6.39375019,4.86250019,4.33750010,3.95000005,3.76250005,3.58750010,3.43125010])
  Aompjeteta['1e6']['init']=np.array([43.7812500,24.2812500,17.0312500,12.6875000,10.3125000,9.40625000,8.93750000])
  Aompjeteta['1e7']={}
  Aompjeteta['1e7']['tps']=np.array([322.093750,167.601562,101.476562,54.3671875,35.1953125,32.0312500,27.2890625])
  Aompjeteta['1e7']['io']=np.array([4.33125019,3.55625010,3.65000010,2.51250005,2.36249995,2.34999990,2.04999995])
  Aompjeteta['1e7']['init']=np.array([80.8750000,49.5312500,38.9687500,28.2187500,24.3750000,22.2187500,21.9062500])
  Aompjeteta['1e8']={}
  Aompjeteta['1e8']['tps']=np.array([2712.10156,1589.63281,962.648438,639.882812,510.617188,471.671875,446.539062])
  Aompjeteta['1e8']['io']=np.array([262.774994,216.431244,193.443756,170.081253,169.668747,161.587494,164.162506])
  Aompjeteta['1e8']['init']=np.array([457.718750,277.468750,201.875000,140.718750,139.906250,138.250000,132.906250])

  # METER
  Aompmemvscmeter={}
  Aompmemvscmeter['1e6']=np.array([8.0,7.9,7.8,7.8,8.0,7.9,8.4,8.3])
  Aompmemvscmeter['1e7']=np.array([11.5,11.6,11.6,11.7,11.7,10.2,10.1,10.5])
  Aompmemvscmeter['1e8']=np.array([46.0,46.0,46.0,46.0,46.1,46.2,46.2,46.4])

  Ampimemvsc={}
  Ampimemvsc['1e6']=np.array([7.0,14.5,21.2,26.0,63.8,124.2,245.3,488.1])
  Ampimemvsc['1e7']=np.array([7.7,13.7,18.5,33.6,64.4,125.2,246.3,489.3])
  Ampimemvsc['1e8']=np.array([14.5,14.3,19.2,41.1,71.6,132.2,253.7,497.7])

  Ampivsc={}
  Ampivsc['1e6']={}
  Ampivsc['1e6']['tot']=np.array([395.875000,349.000000,276.625000,194.125000,118.687500,96.2500000,89.6250000,127.750000])
  Ampivsc['1e7']={}
  Ampivsc['1e7']['tot']=np.array([1054.37500,724.000000,563.875000,433.875000,372.750000,386.312500,344.250000,306.531250])
  Ampivsc['1e8']={}
  Ampivsc['1e8']['tot']=np.array([7348.12500,4164.62500,2884.59766,1550.37500,1050.31250,964.125000,885.500000,971.125000])

  #ETA
  Dompvsceta={}
  Dompvsceta['1e5']={}
  Dompvsceta['1e5']['tps']=np.array([66.0390625,34.7500000,18.6250000,11.2031250,7.92187500,5.35937500,4.29687500,4.07812500,4.62500000])
  Dompvsceta['1e5']['io']=np.array([2.26874995,1.35000002,1.21249998,1.33749998,1.28750002,1.14999998,1.04999995,1.18750000,1.64999998])
  Dompvsceta['1e5']['init']=np.array([31.2500000,17.5000000,11.0625000,8.56250000,7.37500000,5.81250000,5.31250000,5.37500000,6.25000000])
  Dompvsceta['1e6']={}
  Dompvsceta['1e6']['tps']=np.array([555.687500,279.234375,141.500000,73.9062500,45.0000000,24.2968750,15.6093750,10.6025391,9.71875000])
  Dompvsceta['1e6']['io']=np.array([2.15000010,1.63750005,1.32500005,1.41250002,1.33749998,1.18750000,1.29999995,1.76171875,1.76250005])
  Dompvsceta['1e6']['init']=np.array([127.625000,66.3750000,35.2500000,21.2500000,14.6250000,9.62500000,7.62500000,7.25781250,7.37500000])
  Dompvsceta['1e7']={}
  Dompvsceta['1e7']['tps']=np.array([5412.90625,2730.06641,1364.64062,708.281250,415.390625,214.906250,127.515625,71.7656250,64.1098633])
  Dompvsceta['1e7']['io']=np.array([4.55000019,3.48671865,3.01250005,2.71250010,2.82500005,2.84999990,3.23749995,4.71250010,5.23007822])
  Dompvsceta['1e7']['init']=np.array([1092.25000,552.523438,280.187500,146.500000,89.5000000,49.1250000,31.3125000,20.6367188,19.5156250])

  Djet=[1,2,4,8,16,32,40]
  Dompjeteta={}
  Dompjeteta['1e5']={}
  Dompjeteta['1e5']['tps']=np.array([88.1796875,47.8203125,27.6875000,18.6718750,12.5703125,10.0625000,10.0156250])
  Dompjeteta['1e5']['io']=np.array([4.25000000,3.00624990,2.73125005,2.49375010,2.49375010,2.46875000,2.42499995])
  Dompjeteta['1e5']['init']=np.array([47.1875000,29.4375000,20.0625000,15.7812500,14.1562500,13.2812500,12.5625000])
  Dompjeteta['1e6']={}
  Dompjeteta['1e6']['tps']=np.array([695.953125,350.617188,186.054688,100.460938,52.8750000,32.2578125,28.5234375])
  Dompjeteta['1e6']['io']=np.array([4.06874990,3.61249995,3.20000005,3.03749990,2.90000010,2.76250005,2.46250010])
  Dompjeteta['1e6']['init']=np.array([167.250000,91.8125000,50.5000000,32.7812500,22.1875000,17.8750000,15.6250000])
  Dompjeteta['1e7']={}
  Dompjeteta['1e7']['tps']=np.array([6659.34375,3336.52344,1679.60938,857.742188,448.445312,255.945312,222.617188])
  Dompjeteta['1e7']['io']=np.array([8.96875000,7.80625010,6.57499981,6.40625000,5.96875000,5.94999981,5.69999981])
  Dompjeteta['1e7']['init']=np.array([1357.09375,696.312500,378.281250,203.187500,106.468750,66.5000000,59.7500000])

  #1508937 1508927
  Dompmemvsceta={}
  Dompmemvsceta['1e5']=np.array([12.0,11.6,12.1,11.8,11.9,12.0,12.9,14.2])
  Dompmemvsceta['1e6']=np.array([12.2,11.8,11.9,12.0,12.1,12.2,12.8,13.3])
  Dompmemvsceta['1e7']=np.array([14.0,14.0,14.4,14.1,14.3,14.5,15.0,16.1])

  #1508939
  Dompmemvscmeter={}
  Dompmemvscmeter['1e5']=np.array([8.0,8.3,8.3,8.1,8.4,8.5,8.9,9.8])
  Dompmemvscmeter['1e6']=np.array([8.4,8.5,8.5,8.5,8.7,8.7,9.2,10.0])
  Dompmemvscmeter['1e7']=np.array([11.1,10.5,11.1,10.6,10.6,11.1,12.0,16.1])

  # for i in {842749..842774}; do sacct -j $i --format=JobID,MaxVMSize,MaxVMSizeNode,MaxVMSizeTask; done

  Dmpimemvsc={}
  Dmpimemvsc['1e5']=np.array([8.0,15.9,22.3,41.3,79.6,155.9,308.9,615.7])
  Dmpimemvsc['1e6']=np.array([8.1,16.0,22.4,41.4,79.7,156.0,309.0,615.7])
  Dmpimemvsc['1e7']=np.array([8.8,16.8,23.2,42.3,80.4,156.8,309.5,616.4])

  Dmpivsc={}
  Dmpivsc['1e5']={}
  Dmpivsc['1e5']['tot']=np.array([328.843750,198.812500,114.000000,70.2500000,67.1875000,88.6250000,107.000000,154.000000])
  Dmpivsc['1e6']={}
  Dmpivsc['1e6']['tot']=np.array([2763.62500,1445.12500,942.593750,429.484375,217.375000,146.875000,111.953125,142.937500])
  Dmpivsc['1e7']={}
  Dmpivsc['1e7']['tot']=np.array([27089.4375,13841.0000,9207.26562,4035.18750,1905.00000,1128.25000,594.250000,400.207031])

  Eompvsceta_16={}
  Eompvsceta_16['1e5']={}
  Eompvsceta_16['1e5']['tps']=np.array([52.0312500,27.5722656,14.7265625,8.39062500,5.84765625,3.99707031,2.98437500,2.81250000,3.16015625])
  Eompvsceta_16['1e5']['io']=np.array([2.21875000,1.71093750,1.31875002,1.16875005,1.14374995,1.31796873,1.16250002,1.61093748,1.60351562])
  Eompvsceta_16['1e5']['init']=np.array([26.7500000,15.1503906,8.65625000,5.68750000,4.64062500,3.78125000,2.96875000,4.32031250,3.40820312])
  Eompvsceta_16['1e6']={}
  Eompvsceta_16['1e6']['tps']=np.array([325.658203,167.606445,85.7382812,46.1777344,28.4257812,15.7187500,10.1972656,7.89843750,8.09375000])
  Eompvsceta_16['1e6']['io']=np.array([2.78476572,2.08750010,1.65156245,1.46406245,1.49374998,1.12500000,1.50000000,1.61874998,1.81875002])
  Eompvsceta_16['1e6']['init']=np.array([99.3261719,51.6513672,27.0234375,15.3750000,10.4687500,6.40625000,5.09375000,4.20312500,4.64843750])
  Eompvsceta_16['1e7']={}
  Eompvsceta_16['1e7']['tps']=np.array([2784.28125,1425.71875,724.595703,377.183594,231.132812,124.652344,73.5742188,53.9375000,50.4130859])
  Eompvsceta_16['1e7']['io']=np.array([5.62500000,4.53125000,3.81406260,3.78749990,3.74375010,3.39062500,3.65625000,3.90625000,4.06640625])
  Eompvsceta_16['1e7']['init']=np.array([765.281250,388.406250,196.593750,104.984375,63.8750000,35.7578125,21.9843750,15.7812500,15.4375000])

  Eompjeteta={}
  Eompjeteta['1e5']={}
  Eompjeteta['1e5']['tps']=np.array([66.5703125,37.2812500,21.5546875,13.1640625,7.79687500,6.91406250,5.74218750])
  Eompjeteta['1e5']['io']=np.array([4.18125010,3.04375005,3.01874995,4.24375010,2.29375005,2.00624990,2.18125010])
  Eompjeteta['1e5']['init']=np.array([42.3125000,25.0312500,15.4687500,13.4375000,7.93750000,7.31250000,7.09375000])
  Eompjeteta['1e6']={}
  Eompjeteta['1e6']['tps']=np.array([360.429688,192.718750,97.1250000,52.1796875,28.7812500,24.4296875,15.3671875])
  Eompjeteta['1e6']['io']=np.array([4.62500000,3.88124990,3.24375010,3.61249995,2.59375000,2.46250010,2.45000005])
  Eompjeteta['1e6']['init']=np.array([123.968750,67.3125000,35.5625000,24.0625000,13.1562500,12.1562500,9.71875000])
  Eompjeteta['1e7']={}
  Eompjeteta['1e7']['tps']=np.array([2937.46094,1515.60156,759.812500,407.656250,216.679688,181.210938,108.289062])
  Eompjeteta['1e7']['io']=np.array([10.4437504,8.33749962,6.75624990,7.48124981,5.75624990,6.25000000,6.16249990])
  Eompjeteta['1e7']['init']=np.array([823.812500,433.406250,218.500000,116.937500,68.9062500,56.6250000,36.3750000]) 


  #METER
  #1508986 1508989 1509001 1508990 1509002 1509002
  Eompmemvsceta={}
  Eompmemvsceta['1e5']=np.array([11.0,11.0,11.1,11.2,11.3,11.4,11.8,12.0])
  Eompmemvsceta['1e6']=np.array([11.3,11.3,12.1,11.5,11.6,11.8,12.6,13.9])
  Eompmemvsceta['1e7']=np.array([13.9,13.5,13.6,13.7,13.8,14.2,15.1,16.8])

  #1509003
  Eompmemvscmeter={}
  Eompmemvscmeter['1e5']=np.array([8.0,8.3,8.3,8.1,8.5,8.5,8.9,9.8])
  Eompmemvscmeter['1e6']=np.array([8.4,8.5,8.5,8.5,8.7,8.7,9.2,10.0])
  Eompmemvscmeter['1e7']=np.array([11.1,10.5,11.1,10.6,11.4,11.1,12.0,13.7])

  Empimemvsc={}
  Empimemvsc['1e5']=np.array([7.0,13.6,18.5,33.6,64.1,124.9,246.6,491.1])
  Empimemvsc['1e6']=np.array([7.0,13.7,18.6,33.7,64.2,125.1,246.9,491.5])
  Empimemvsc['1e7']=np.array([7.7,14.4,19.3,34.5,65.0,126.1,248.0,492.5])
                                                                                                                          
  Eompvsc={}
  Eompvsc['1e5']={}
  Eompvsc['1e5']['tps']=np.array([49.9687500,39.0000000,28.6220703,26.2500000,24.5000000,13.9531250,10.5312500,7.96875000])
  Eompvsc['1e5']['io']=np.array([2.42499995,1.60000002,1.59765625,1.52499998,1.77499998,1.27812505,1.66250002,1.89999998])
  Eompvsc['1e5']['init']=np.array([41.5000000,33.8750000,47.9179688,9.50000000,8.12500000,6.46875000,7.37500000,6.00000000])
  Eompvsc['1e6']={}
  Eompvsc['1e6']['tps']=np.array([307.757812,270.785156,225.187500,205.375000,202.226562,111.203125,62.0117188,41.0937500])
  Eompvsc['1e6']['io']=np.array([2.58124995,2.69765615,2.34999990,1.77499998,1.75625002,2.37500000,1.77187502,2.09999990])
  Eompvsc['1e6']['init']=np.array([105.875000,94.0937500,139.625000,56.8125000,81.4921875,30.2500000,20.2812500,15.6250000])
  Eompvsc['1e7']={}
  Eompvsc['1e7']['tps']=np.array([2643.25000,2437.14062,2043.35938,2029.97461,1967.75000,1059.26172,607.500000,334.375000])
  Eompvsc['1e7']['io']=np.array([6.02500010,6.69999981,7.34999990,5.36875010,7.58750010,5.77226543,7.97499990,5.80000019])
  Eompvsc['1e7']['init']=np.array([752.375000,685.312500,550.312500,588.781250,530.312500,274.210938,160.750000,110.750000])

  Empivsc={}
  Empivsc['1e5']={}
  Empivsc['1e5']['tot']=np.array([205.500000,135.562500,75.8750000,60.1875000,57.3750000,75.5000000,87.2500000,128.375000])
  Empivsc['1e6']={}
  Empivsc['1e6']['tot']=np.array([1131.87500,665.500000,457.000000,242.593750,132.687500,98.9375000,88.8750000,125.875000])
  Empivsc['1e7']={}
  Empivsc['1e7']['tot']=np.array([9192.62500,4758.09375,3199.75000,1500.97656,821.750000,545.625000,333.812500,241.843750])

  kleuren=['k','cornflowerblue','forestgreen','coral']

  ompproc=np.array([1,2,4,8,16,32,64,128])#,80])
  ompproc2=np.array([1,2,4,8,16,32,64,128,256])
  mpiproc=np.array([1,2,4,8,16,20,40])

  print('A')
  bibomp1 = Aompjeteta
  bibomp2 = Aompvsceta
  proc=mpiproc
  for key1 in bibomp1.keys():
    interp_bibomp=interp1d(Djet,(bibomp1[key1]['tps'][:]-bibomp1[key1]['io'][:])/((bibomp1[key1]['tps'][0]-bibomp1[key1]['io'][0])/(Djet)))
    print('Jet',key1,interp_bibomp(32))

  for key1 in bibomp2.keys():
    print('VSC',key1,(bibomp2[key1]['tps'][5]-bibomp2[key1]['io'][5])/((bibomp2[key1]['tps'][0]-bibomp2[key1]['io'][0])/(ompproc2[5])))

  key0='1e6'
  key1='1e7'
  key2='1e8'
  interp_bibomp=interp1d(ompproc2,bibomp2[key1]['tps'])
  print(key1,'weak VSC:', (interp_bibomp(10)-(bibomp2[key0]['tps'][0]))/(bibomp2[key0]['tps'][0]))

  interp_bibomp=interp1d(ompproc2,bibomp2[key2]['tps'])
  print(key2,'weak VSC:', (interp_bibomp(10)-(bibomp2[key1]['tps'][0]))/(bibomp2[key1]['tps'][0]))

  interp_bibomp=interp1d(Djet,bibomp1[key1]['tps'][:])
  print(key1,'weak Jet:', (interp_bibomp(10)-(bibomp1[key0]['tps'][0]))/(bibomp1[key0]['tps'][0]))

  interp_bibomp=interp1d(Djet,bibomp1[key2]['tps'][:])
  print(key2,'weak Jet:', (interp_bibomp(10)-(bibomp1[key1]['tps'][0]))/(bibomp1[key1]['tps'][0]))

  print(key1,'weak VSC:', (bibomp2[key1]['tps'][0]-(bibomp2[key0]['tps'][0])*10)/(bibomp2[key0]['tps'][0]*10))
  print(key2,'weak VSC:', (bibomp2[key2]['tps'][0]-(bibomp2[key1]['tps'][0])*10)/(bibomp2[key1]['tps'][0]*10))
  print(key1,'weak Jet:', (bibomp1[key1]['tps'][1]-(bibomp1[key0]['tps'][1])*10)/(bibomp1[key0]['tps'][1]*10))
  print(key2,'weak Jet:', (bibomp1[key2]['tps'][1]-(bibomp1[key1]['tps'][1])*10)/(bibomp1[key1]['tps'][1]*10))

  print('E')
  bibomp1 = Eompjeteta
  bibomp2 = Eompvsceta_16
  proc=mpiproc
  for key1 in bibomp1.keys():
    interp_bibomp=interp1d(Djet,(bibomp1[key1]['tps'][:]-bibomp1[key1]['io'][:])/((bibomp1[key1]['tps'][0]-bibomp1[key1]['io'][0])/(Djet)))
    print('Jet',key1,interp_bibomp(32))

  for key1 in bibomp2.keys():
    print('VSC',key1,(bibomp2[key1]['tps'][5]-bibomp2[key1]['io'][5])/((bibomp2[key1]['tps'][0]-bibomp2[key1]['io'][0])/(ompproc2[5])))

  key0='1e5'
  key1='1e6'
  key2='1e7'
  interp_bibomp=interp1d(ompproc2,bibomp2[key1]['tps'])
  print(key1,'weak VSC:', (interp_bibomp(10)-(bibomp2[key0]['tps'][0]))/(bibomp2[key0]['tps'][0]))

  interp_bibomp=interp1d(ompproc2,bibomp2[key2]['tps'])
  print(key2,'weak VSC:', (interp_bibomp(10)-(bibomp2[key1]['tps'][0]))/(bibomp2[key1]['tps'][0]))

  interp_bibomp=interp1d(Djet,bibomp1[key1]['tps'][:])
  print(key1,'weak Jet:', (interp_bibomp(10)-(bibomp1[key0]['tps'][0]))/(bibomp1[key0]['tps'][0]))

  interp_bibomp=interp1d(Djet,bibomp1[key2]['tps'][:])
  print(key2,'weak Jet:', (interp_bibomp(10)-(bibomp1[key1]['tps'][0]))/(bibomp1[key1]['tps'][0]))

  print(key1,'weak VSC:', (bibomp2[key1]['tps'][0]-(bibomp2[key0]['tps'][0])*10)/(bibomp2[key0]['tps'][0]*10))
  print(key2,'weak VSC:', (bibomp2[key2]['tps'][0]-(bibomp2[key1]['tps'][0])*10)/(bibomp2[key1]['tps'][0]*10))
  print(key1,'weak Jet:', (bibomp1[key1]['tps'][1]-(bibomp1[key0]['tps'][1])*10)/(bibomp1[key0]['tps'][1]*10))
  print(key2,'weak Jet:', (bibomp1[key2]['tps'][1]-(bibomp1[key1]['tps'][1])*10)/(bibomp1[key1]['tps'][1]*10))

  print('D')
  bibomp1 = Dompjeteta
  bibomp2 = Dompvsceta
  proc=Djet
  for key1 in bibomp1.keys():
    interp_bibomp=interp1d(Djet,(bibomp1[key1]['tps'][:]-bibomp1[key1]['io'][:])/((bibomp1[key1]['tps'][0]-bibomp1[key1]['io'][0])/(Djet)))
    print('Jet',key1,interp_bibomp(32))

  for key1 in bibomp2.keys():
    print('VSC',key1,(bibomp2[key1]['tps'][5]-bibomp2[key1]['io'][5])/((bibomp2[key1]['tps'][0]-bibomp2[key1]['io'][0])/(ompproc2[5])))

  key0='1e5'
  key1='1e6'
  key2='1e7'
  interp_bibomp=interp1d(ompproc2,bibomp2[key1]['tps'])
  print(key1,'weak VSC:', (interp_bibomp(10)-(bibomp2[key0]['tps'][0]))/(bibomp2[key0]['tps'][0]))

  interp_bibomp=interp1d(ompproc2,bibomp2[key2]['tps'])
  print(key2,'weak VSC:', (interp_bibomp(10)-(bibomp2[key1]['tps'][0]))/(bibomp2[key1]['tps'][0]))

  interp_bibomp=interp1d(Djet,bibomp1[key1]['tps'][:])
  print(key1,'weak Jet:', (interp_bibomp(10)-(bibomp1[key0]['tps'][0]))/(bibomp1[key0]['tps'][0]))

  interp_bibomp=interp1d(Djet,bibomp1[key2]['tps'][:])
  print(key2,'weak Jet:', (interp_bibomp(10)-(bibomp1[key1]['tps'][0]))/(bibomp1[key1]['tps'][0]))

  print(key1,'weak VSC:', (bibomp2[key1]['tps'][0]-(bibomp2[key0]['tps'][0])*10)/(bibomp2[key0]['tps'][0]*10))
  print(key2,'weak VSC:', (bibomp2[key2]['tps'][0]-(bibomp2[key1]['tps'][0])*10)/(bibomp2[key1]['tps'][0]*10))
  print(key1,'weak Jet:', (bibomp1[key1]['tps'][1]-(bibomp1[key0]['tps'][1])*10)/(bibomp1[key0]['tps'][1]*10))
  print(key2,'weak Jet:', (bibomp1[key2]['tps'][1]-(bibomp1[key1]['tps'][1])*10)/(bibomp1[key1]['tps'][1]*10))

  ompproc=np.array([1,2,4,8,16,32,64,128])#,80])
  ompproc2=np.array([1,2,4,8,16,32,64,128])
  mpiproc=np.array([1,2,4,8,16,20,40])

  fig,ax=plt.subplots(1,figsize=(9,7))

  bibomp={}
  bibomp[0]=Aompmemvscmeter
  bibomp2={}
  bibomp2[0]=Aompmemvsceta
  bibompvsc={}
  bibompvsc[0]=Ampimemvsc
  kleuren=['indigo','orchid','slategray']
  lws=[4,3,2]
  i=0
  for key1 in bibomp[0].keys():

    ax.set_xscale('log')
    if i==0:
      ax.text(0.97,.97,'Case Tracer',horizontalalignment='right',verticalalignment='top',
        transform=ax.transAxes,weight='bold',color='darkblue',fontsize=25)
      ax.set_facecolor((0.0, 0.0, 0.5450980392156862, 0.03))

    ax.plot(ompproc,bibomp[0][key1],c=kleuren[i],lw=lws[i],ls='-',label='N$_{\\rm part}$='+key1)
    ax.plot(ompproc,bibomp2[0][key1],c=kleuren[i],lw=lws[i],ls=':')
    ax.plot(ompproc2,bibompvsc[0][key1],c=kleuren[i],lw=lws[i],ls='--', alpha=1)

    i += 1

  ax.grid(True,alpha=0.5)
  ax.set_xticks((ompproc2).astype(int))
  ax.get_xaxis().set_major_formatter(mpl.ticker.ScalarFormatter())
  ax.set_xlabel('$N_{\\rm core}$',fontsize=28)
  ax.set_ylabel('Memory (Gb)',fontsize=28)
  ax.set_ylim(bottom=0)
  ax.set_xlim([1,128])
  #ax.legend(loc='upper right',fontsize=22)

  keys=list(bibomp[0].keys())
  lines = [Line2D([0],[0],color=kleuren[i],lw=lws[i],ls='-',label='N$_{\\rm part}$='+keys[i]) for i in range(3)]
  line1 = Line2D([0],[0],color='k',label='FLEXPART-11: $z$',ls='-',lw=3)
  line1a = Line2D([0],[0],color='k',label='FLEXPART-11: $\\eta$',ls=':',lw=3)
  line2 = Line2D([0],[0],label='FLEXPART-10.4',color='k',ls='--',lw=2,alpha=1)
  ax.legend(handles=lines+[line1,line1a,line2],frameon=True,loc='upper left',fontsize=22)

  fig.tight_layout()
  #plt.tight_layout()
  plt.savefig('OPTetamem.png')
  plt.savefig('OPTetamem.pdf')
  plt.close()